﻿/*SIED : Significant Isotopic Enrichment Detection
version 1.71
28 jan 2015
Copyright (C) 2015
Anthony Delaune
Camille Ripoll
Armelle Cabin-Flaman
Laboratoire GPM UMR CNRS 6634 Equipe NanoCARE
Université de Rouen
anthony.delaune@univ-rouen.fr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <stdio.h>
#include <mpi.h>
#include <math.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>
#include <time.h>
#include <string.h>

double seuilisot(double i, float moyenne, double rapport)
{
// Compute the p-values
unsigned int k,l,m;
double probap[100001]={0};
double pcumul=0;
int nmin14, nmin15, nmax14, nmax15;


    nmin14=(int) floor(i-15*sqrt(i));
    if (nmin14<0) 
    {nmin14=0;}
    nmin15=(int) floor((moyenne*i/(1-moyenne)-15*sqrt(moyenne*i/(1-moyenne))));
    if (nmin15<0) 
    {
        nmin15=0;
        }
    
    nmax14=(int) ceil(i+15*sqrt(i));
    nmax15=(int) ceil((moyenne*i/(1-moyenne)+15*sqrt(moyenne*i/(1-moyenne))));
    if (nmax15<5)
    {
    nmax15=5;
    }

    if (nmax14<5)
    {
    nmax14=5;
    }

    
    for (k=nmin14;k<nmax14;k++)
    {
        for (l=nmin15; l<nmax15;l++)
        {
            if ((k+l)==0)
             {
              probap[0]+=gsl_ran_poisson_pdf(k,i)*gsl_ran_poisson_pdf(l,i*(moyenne/(1-moyenne)));
              }
            else
            {
             m=(int)ceil((float)(100000*l/(l+k)));
             probap[m]+=gsl_ran_poisson_pdf(k,i)*gsl_ran_poisson_pdf(l,i*(moyenne/(1-moyenne)));
            }
        }
    }
     pcumul=0;
     for (m=0;m<(100000*rapport);m++)
     {
         pcumul+=probap[m];
     }
  return (double)(1-pcumul);
      
}

int main(int argc, char *argv[])
{

 time_t temps_debut, temps_fin;
 double duree;
 int tid,nthreads;
  FILE *config;              
  int raster_line, raster_col;       
  int filtretype;
  int two_tailed;
  char nom_fich[62];
  
  char nom_num[62];
  char nom_denom[62];
  char nom_norme[62];
  char nom_temp[62];         
  char *nom_config;

  FILE *fich_input; 
  FILE *fich_out_rap;
  FILE *fich_out_pva;
  FILE *fich_out_red;
  FILE *fich_out_lam;
  FILE *fich_out_fin;
  
  FILE *fich_out_rap_pos;
  FILE *fich_out_rap_first;
  FILE *fich_out_num_pos;
  FILE *fich_out_num_first;
  FILE *fich_out_denom_pos;
  FILE *fich_out_denom_first;
  FILE *fich_out_num;
  FILE *fich_out_denom;
  
  
  long int buff; 
  int length;
            
  unsigned int mindenom=1001, maxdenom=0;            
  unsigned int seuildenom;
  unsigned int seuilmoy; 
  float moyratio;
  float tresh=0.00001;
  int loopmin, loopmax;
  int k,l, inf, sup, optiminf, optimsup, nbpxinf, nbpxsup ;
  double progres, proba, probamax, probacuminf, probacumsup, probamaxinf, probamaxsup;
  float shift, tempinf, tempsup;
  int i,j;
  int nbpxtemp=0, nbpx=0; //number of pixels analyzed
  int nbbleu=0, nbrouge=0; //positive pixel counter
  int nb_inf_bonf=0, nb_inf_seuil=0, nb_sup_bonf=0, nb_sup_seuil=0;//positive pixel counter two-tailed
 temps_debut = time(NULL);

  MPI_Init(&argc,&argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &tid);
  MPI_Comm_size(MPI_COMM_WORLD, &nthreads);


/*for the master process*/
if (tid ==0)
{

	/*config reading by master thread*/
  printf("SIED 1.71\n 28 jan 2015\n");
printf("Syntax example:\n mpiexec -n 2 SIED config.ini\n where config.ini contains the parameters of analysis\nWARNING : All the files to be analyzed MUST be in the current directory!\n");

if (argc==1)
{
printf("Error !\n Arguments lacking !\n");
MPI_Finalize();
return(0);
}

if (argc>1)
{
nom_config=argv[1];
}

   config=fopen(nom_config, "r");
   if (config==NULL)
   {
   printf("Error while reading %s\nVerify %s is in the current directory.\nProgram exit.\n", nom_config, nom_config);
   MPI_Finalize();
   return(0);	   
   }
   
   printf("Reading of %s\n", nom_config);
   
   fscanf(config, "%d\n", &raster_line); 
   printf("Raster (lines): %d\n", raster_line);
   fscanf(config, "%d\n", &raster_col); 
   printf("Raster (columns): %d\n", raster_col);
   
   fscanf(config, "%d\n", &filtretype);   
   printf("Type of the lambda filter : %d\n", filtretype);
 

	if ((filtretype<0)+(filtretype>4))
	{
	printf("Unvalid type of lambda filter. The default value = 3 (matrix 7x7) is used\n");
	filtretype=3;
	}
	for (i=1; i<nthreads; i++)
	{
	MPI_Rsend(&raster_line, 1, MPI_INT, i, i, MPI_COMM_WORLD);
	MPI_Rsend(&raster_col, 1, MPI_INT, i, i, MPI_COMM_WORLD);
	MPI_Rsend(&filtretype, 1, MPI_INT, i, i, MPI_COMM_WORLD);
	}
}
 if (tid!=0)
{
	MPI_Recv(&raster_line, 1, MPI_INT, 0, tid, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	MPI_Recv(&raster_col, 1, MPI_INT, 0, tid, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	MPI_Recv(&filtretype, 1, MPI_INT, 0, tid, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
}

	MPI_Barrier(MPI_COMM_WORLD);
unsigned int* num=(unsigned int*) calloc(raster_line*raster_col, sizeof(unsigned int));
unsigned int* denom=(unsigned int*) calloc(raster_line*raster_col, sizeof(unsigned int));
float* lambda=(float*) calloc(raster_line*raster_col, sizeof(float));
double* rapport=(double *) calloc(raster_line*raster_col, sizeof(double));
unsigned int* normasse=(unsigned int*) calloc(raster_line*raster_col, sizeof(unsigned int));
double* pvalue=(double*) calloc(raster_line*raster_col, sizeof(double));

 if (tid ==0)
{
   
   fscanf(config, "%s", &nom_num); 
   
   printf("File %s ", nom_num);
   fich_input=fopen(nom_num, "r");  
   if (fich_input==NULL)
   {
   printf("Error while reading %s\nVerify your config file.\nProgram exit.\n", nom_num);
   MPI_Finalize();
   return(0);	   
   }
     
   fflush(fich_input);
   
   printf("OK\n");
   
   
   for (i=0; i<raster_line; i++) 
       {
       for (j=0; j<raster_col; j++) 
           {
           fscanf(fich_input, "%d", &num[i*raster_col+j]);           
           }
       }
   fclose(fich_input);              
   fscanf(config, "%s", &nom_denom); 
   fich_input=fopen(nom_denom, "r"); 
      if (fich_input==NULL)
   {
   printf("Error while reading %s\nVerify your config file.\nProgram exit.\n", nom_denom);
   MPI_Finalize();
   return(0);	   
   }
 
   fflush(fich_input);
   printf("File %s OK\n",nom_denom);
/*/we complete the denominator table*/
   
   for (i=0; i<raster_line; i++) 
       {
       for (j=0; j<raster_col; j++) 
           {
           fscanf(fich_input, "%d", &denom[i*raster_col+j]);
	   if (denom[i*raster_col+j]>maxdenom)
              {
              maxdenom=denom[i*raster_col+j];
              }
           if (denom[i*raster_col+j]<mindenom)
              {
              mindenom=denom[i*raster_col+j];
              }
           }
       }
       
   fclose(fich_input);        
   fscanf(config, "%d", &seuildenom); 
   printf("Threshold of analysis : %d\n ", seuildenom);
   fscanf(config, "%d", &seuilmoy); 
   printf("Threshold of fraction averaging : %d\n ", seuilmoy);
   
   if (mindenom<seuildenom)
   {
                           mindenom=seuildenom;
   }       

/*standard file*/   
   fscanf(config, "%s", &nom_norme); 
   
   printf("File %s ", nom_norme);
   
   fich_input=fopen(nom_norme, "r");    
      if (fich_input==NULL)
   {
   printf("Error while reading %s\nVerify your config file.\nProgram exit.\n", nom_norme);
   MPI_Finalize();
   return(0);	   
   }

   fflush(fich_input);
   
   printf("OK\n");
   
/*/we complete the standard file*/
   
   for (i=0; i<raster_line; i++) 
       {
       for (j=0; j<raster_col; j++) 
           {
           fscanf(fich_input, "%d", &normasse[i*raster_col+j]);
                  
           
           }
       }
   fclose(fich_input);              
   
   fscanf(config, "%f", &moyratio); 
   printf("Reference isotopic fraction : %lf\n", moyratio);
   fscanf(config, "%f", &tresh);
   printf("Alternative probability threshold : %f\n", tresh);

   fscanf(config, "%d", &two_tailed);
   printf("Statistical test two_tailed= %d : ", two_tailed);
   if (two_tailed)
   {
	printf("Heterogeneity study\n");
   }
   else {
	printf("Enrichment study\n");
   }

   fscanf(config, "%s", &nom_temp); 
   printf("Name of analysis :%s\n", nom_temp);

   fclose(config); 
   /*end initialization of files by the master process*/
/*sends numerator and denominator tables*/
	for (i=1; i<nthreads; i++)
	{
	MPI_Rsend(&seuildenom, 1, MPI_UNSIGNED, i, i, MPI_COMM_WORLD);
	MPI_Rsend(&*denom, raster_line*raster_col, MPI_UNSIGNED, i, i, MPI_COMM_WORLD);
	MPI_Rsend(&*num, raster_line*raster_col, MPI_UNSIGNED, i, i, MPI_COMM_WORLD);
	}

}

 if (tid!=0)
{
	MPI_Recv(&seuildenom, 1, MPI_UNSIGNED, 0, tid, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	MPI_Recv(&*denom, raster_line*raster_col, MPI_UNSIGNED, 0, tid, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	MPI_Recv(&*num, raster_line*raster_col, MPI_UNSIGNED, 0, tid, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
}

	MPI_Barrier(MPI_COMM_WORLD);



 loopmin=0;
 loopmax=raster_line;
 
/* parallel process : lambda parameters determination*/
if (filtretype>0)
{
for (i=loopmin; i<loopmax; i+=nthreads) 
{
 if (((i+tid)>(filtretype-1))*((i+tid)<(raster_line-filtretype))) /*if we're on the right lines*/
 { 
  for (j=filtretype; j<(raster_col-filtretype); j++) /*columns*/
  {
  /*for the pixel line i and column j*/
  
//test if the pixel must be analyzed
      if ((denom[(i+tid)*raster_col+j])>=seuildenom)
      {
	
	// calculation of the sum of probabilities with lambda=denom i,j
	probamax=0;
	optiminf=1;
	optimsup=1;
	for (k=-filtretype; k<(filtretype+1); k++)
	{
	  for (l=-filtretype; l<(filtretype+1); l++)
	  {
	    probamax+=gsl_ran_poisson_pdf(denom[((i+tid)+k)*raster_col+j+l], denom[(i+tid)*raster_col+j]);
	  }
	}
	//searching for optimum at left	
	progres=1;
	tempinf=denom[(i+tid)*raster_col+j];
	tempsup=denom[(i+tid)*raster_col+j];
	shift=-1;
	probamaxinf=probamax;
	probamaxsup=probamax;
	while ((progres>0)*(tempinf>1)) 
	{
	  
	  tempinf=tempinf+shift;
	  proba=0;
	  for (k=-filtretype; k<(filtretype+1); k++)
	  {
	    for (l=-filtretype; l<(filtretype+1); l++)
	    {
	      proba+=gsl_ran_poisson_pdf(denom[((i+tid)+k)*raster_col+j+l], tempinf); //the sum of the probabilities of the matrix is computed
	    }
	  }
	  progres=proba-probamaxinf;
	  if (progres>0)
	    {
	    probamaxinf=proba; //if progress is positive otherwise it's the end of the loop
	    optiminf++;
	    }
	}
	// searching for optimum at right
	progres=1;
	shift=1;
	while (progres>0) 
	{
	  
	  tempsup=tempsup+shift;
	  proba=0;
	  for (k=-filtretype; k<(filtretype+1); k++)
	  {
	    for (l=-filtretype; l<(filtretype+1); l++)
	    {
	      proba+=gsl_ran_poisson_pdf(denom[((i+tid)+k)*raster_col+j+l], tempsup); //the sum of the probabilities of the matrix is computed
	    }
	  }
	  progres=proba-probamaxsup;
	  if (progres>0)
	    {
	    probamaxsup=proba; //if progress is positive otherwise it's the end of the loop
	    optimsup++;
	    }
	}
	
	probacuminf=gsl_cdf_poisson_P(denom[(i+tid)*raster_col+j], tempinf);
	probacumsup=gsl_cdf_poisson_P(denom[(i+tid)*raster_col+j], tempsup);
	// Kolmogorov test at least one pixel must be compatible with the distribution of the lambda found
	nbpxinf=0;
	nbpxsup=0;
	  for (k=-filtretype; k<(filtretype+1); k++)
	  {
	    for (l=-filtretype; l<(filtretype+1); l++)
	    {
		proba=gsl_ran_poisson_pdf(denom[((i+tid)+k)*raster_col+j+l], tempsup);
	      if ((0.00005<proba)*(proba<0.99995))
		{nbpxsup++;}
		proba=gsl_ran_poisson_pdf(denom[((i+tid)+k)*raster_col+j+l], tempinf);
	      if ((0.00005<proba)*(proba<0.99995))
		{nbpxinf++;}
	    }
	  }

	inf=(optiminf>0)&&(probacuminf>0.00005)&&(probacuminf<0.99995)*(nbpxinf>1);
	sup=(optimsup>0)&&(probacumsup>0.00005)&&(probacumsup<0.99995)*(nbpxsup>1);
	if (inf&&sup) // if both optima are compatible with the table i,j
	{
	   if (probamaxinf>=probamaxsup)
	    {
	    lambda[(i+tid)*raster_col+j]=tempinf; //the optimum value is given to lambda
	    }
	   if (probamaxinf<probamaxsup)
	    {
	    lambda[(i+tid)*raster_col+j]=tempsup; //the optimum value is given to lambda
	    }

	}
	else
	{
	  if (inf) //if the lower optimum is compatible with table i,j
	    {
	      lambda[(i+tid)*raster_col+j]=tempinf; //the optimum value is given to lambda
	    }
	  if (sup) //if the higher optimum is compatible with table i,j
	    {
	      lambda[(i+tid)*raster_col+j]=tempsup; //the optimum value is given to lambda
	    }
      
	  if (((inf) || (sup))==0)
	  {
	  lambda[(i+tid)*raster_col+j]=-1;  
	  }
	}
      } //end of : If pixel analyzed
  
  }
 }

}
}

if (filtretype==0)
{
for (i=loopmin; i<(loopmax); i+=nthreads) /*lines according to process distribution*/
{
  for (j=filtretype; j<(raster_col-filtretype); j++) /*columns*/
  {
  /*for the pixel line i and column j*/
  lambda[(i+tid)*raster_col+j]=denom[(i+tid)*raster_col+j];
  }
 }
 
}



float* lambdatemp=(float*) calloc(raster_line*raster_col, sizeof(float));
MPI_Barrier(MPI_COMM_WORLD);

MPI_Reduce(lambda, lambdatemp, raster_line*raster_col, MPI_FLOAT, MPI_MAX, 0,MPI_COMM_WORLD); 
 

/*if the average is not given by the config file, it must be computed*/
/*if the average is given, the isotopic fractions will be computed later*/
  MPI_Barrier(MPI_COMM_WORLD);


if (tid==0)
{
	
  for (i=1; i<nthreads; i++)
  {
   MPI_Rsend(&moyratio, 1, MPI_DOUBLE, i, i, MPI_COMM_WORLD);
   MPI_Rsend(&seuilmoy, 1, MPI_UNSIGNED, i, i, MPI_COMM_WORLD);
  } 
   
}
if (tid!=0)
{
   MPI_Recv(&moyratio, 1, MPI_DOUBLE, 0, tid, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
   MPI_Recv(&seuilmoy, 1, MPI_UNSIGNED, 0, tid, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
}

/*the pixel to pixel ratio is computed*/
probacuminf=0.0;	//recycling of this var to compute average isotopic fraction
probacumsup=0.0;	//recycling of this var to compute average isotopic fraction
nbpxsup=0;		//recycling of this var to count pixels 
nbpxinf=0;		//recycling of this var to count pixels >seuilmoy

for (i=loopmin; i<(loopmax); i+=nthreads) /*lines according to process distribution*/
{
 if (((i+tid)>(filtretype-1))*((i+tid)<(raster_line-filtretype+1))) /*if we're on the right lines*/
 { 
  for (j=filtretype; j<(raster_col-filtretype+1); j++) /*columns*/
  {
	rapport[(i+tid)*raster_col+j]=(double)(num[(i+tid)*raster_col+j])/(num[(i+tid)*raster_col+j]+denom[(i+tid)*raster_col+j]);
	if ((denom[(i+tid)*raster_col+j])>seuilmoy)
	{
	 probacumsup+=rapport[(i+tid)*raster_col+j];
	 nbpxsup++;
	}
  }
 }
}

/*the information must be sent to all processes*/
MPI_Allreduce(&probacumsup, &probacuminf, 1, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD); 
MPI_Allreduce(&nbpxsup, &nbpxinf, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD); 

if (moyratio<0.0)
{
moyratio=(double)probacuminf/nbpxinf;  //the moyratio is replaced by the measured value if <0
}
probacuminf=(double)probacuminf/nbpxinf; //we stock the real report

printf("Computation of lambda parameters DONE.\nAverage isotopic fraction is %.9f\nComputation of p-values\n", probacuminf);



/*parallel calculation of p-values*/
for (i=loopmin; i<(loopmax); i+=nthreads) /*lines according to process distribution*/
{
 if (((i+tid)>(filtretype-1))*((i+tid)<(raster_line-filtretype))) /*if we're in the right lines*/
 { 
  for (j=filtretype; j<(raster_col-filtretype); j++) /*columns*/
  {
  /*for a pixel line i and column j*/
//test if the pixel must be analyzed
      if (((denom[(i+tid)*raster_col+j])>=seuildenom)*((lambda[(i+tid)*raster_col+j])>0)) //if the lambda exists
      {
	nbpxtemp++;
	pvalue[(i+tid)*raster_col+j]=seuilisot(lambda[(i+tid)*raster_col+j], moyratio, rapport[(i+tid)*raster_col+j]);
      }

   }
  }

printf("Thread %d has calculated the row %d\n",tid, (i+tid));  
}
MPI_Barrier(MPI_COMM_WORLD);
MPI_Reduce(&nbpxtemp, &nbpx, 1, MPI_INT, MPI_SUM, 0,MPI_COMM_WORLD);
double* pvaluetemp=(double*) calloc(raster_line*raster_col, sizeof(double));
MPI_Barrier(MPI_COMM_WORLD);
MPI_Reduce(pvalue, pvaluetemp, raster_line*raster_col, MPI_DOUBLE, MPI_SUM, 0,MPI_COMM_WORLD);
double* rapporttemp=(double*) calloc(raster_line*raster_col, sizeof(double));
MPI_Barrier(MPI_COMM_WORLD);
MPI_Reduce(rapport, rapporttemp, raster_line*raster_col, MPI_DOUBLE, MPI_SUM, 0,MPI_COMM_WORLD);


if (tid==0)
{
printf("Analysis completed, saving files.\n");
printf("%s\n", nom_temp);
sprintf(nom_fich, "%s_summary.txt", nom_temp);
printf("%s\n", nom_fich);
fich_out_fin=fopen(nom_fich, "w");

fprintf(fich_out_fin, "SIED analysis report (v1.71 28 jan 2015)\n");
fprintf(fich_out_fin, "Analysis %s\n", nom_temp);
   if (two_tailed)
   {
	fprintf(fich_out_fin,"Two-tailed statistical test : Heterogeneity study\n");
   }
   else {
	fprintf(fich_out_fin,"One-tailed statistical test : Enrichment study\n");
   }

fprintf(fich_out_fin, "Analysis threshold=%d\nFraction averaging threshold=%d\nReference isotopic fraction=%f\nalternative probability threshold (second class pixels)=%f\n", mindenom, seuilmoy, moyratio, tresh);
fprintf(fich_out_fin, "Extension of the matrix for lambda determination (filter type)=%d\n\nCalculated average isotopic fraction=%.9lf\nNumber of computed pixels for isotopic fraction calculation=%d\nNumber of analyzed pixels=%d\n", filtretype, probacuminf, nbpxinf, nbpx);

sprintf(nom_fich, "%s_fraction.txt", nom_temp);
fich_out_rap=fopen(nom_fich, "w");
printf("%s\n", nom_fich);

sprintf(nom_fich, "%s_pvalue.txt", nom_temp);
fich_out_pva=fopen(nom_fich, "w");
printf("%s\n", nom_fich);

sprintf(nom_fich, "%s_image_out.bmp", nom_temp);
fich_out_red=fopen(nom_fich, "wb");
printf("%s\n", nom_fich);

sprintf(nom_fich, "%s_lambda.txt", nom_temp);
fich_out_lam=fopen(nom_fich, "w");
printf("%s\n", nom_fich);

sprintf(nom_fich, "%s_positive_fraction.txt", nom_temp);
fich_out_rap_pos=fopen(nom_fich, "w");
printf("%s\n", nom_fich);

sprintf(nom_fich, "%s_positive_class1_fraction.txt", nom_temp);
fich_out_rap_first=fopen(nom_fich, "w");
printf("%s\n", nom_fich);

/*correction of nom_num and nom_denom to keep only the mass name*/
/*addition of a If for compatibility with previous SIED maker*/
length=strlen(nom_temp);

sprintf(nom_fich, "");

if (nom_num[0]==nom_temp[0])
{
	for (i=length; i<(strlen(nom_num)+1); i++)
	{
	nom_fich[i-length]=nom_num[i];

	}


	sprintf(nom_num, "%s", nom_fich);
}
	printf("%s\n", nom_num);



sprintf(nom_fich, "");

if (nom_denom[0]==nom_temp[0])
{
for (i=length; i<(strlen(nom_denom)+1); i++)
{
nom_fich[i-length]=nom_denom[i];
}
sprintf(nom_denom, "%s", nom_fich);
}
printf("%s\n", nom_denom);




sprintf(nom_fich, "%s_positive_%s", nom_temp, nom_num);

fich_out_num_pos=fopen(nom_fich, "w");

sprintf(nom_fich, "%s_positive_class1_%s", nom_temp, nom_num);
fich_out_num_first=fopen(nom_fich, "w");

sprintf(nom_fich, "%s_positive_%s", nom_temp, nom_denom);
fich_out_denom_pos=fopen(nom_fich, "w");

sprintf(nom_fich, "%s_positive_class1_%s", nom_temp, nom_denom);
fich_out_denom_first=fopen(nom_fich, "w");

sprintf(nom_fich, "%s_crop_%s", nom_temp, nom_num);
fich_out_num=fopen(nom_fich, "w");

sprintf(nom_fich, "%s_crop_%s", nom_temp, nom_denom);
fich_out_denom=fopen(nom_fich, "w");



// header bmp
for (k=0; ((raster_col-2*filtretype+k)%4)!=0; k++); //k is calculated to fill the bmp
buff=19778;
fwrite(&buff, 2, 1, fich_out_red); //BM
buff=54+(raster_line-2*filtretype)*((raster_col-2*filtretype)*3+k);
fwrite(&buff, 4, 1, fich_out_red); //bmp file size
buff=0;
fwrite(&buff, 4, 1, fich_out_red);  //reserved
buff=54;
fwrite(&buff, 4, 1, fich_out_red); //offset
buff=40;
fwrite(&buff, 4, 1, fich_out_red);  //windows bmp type
buff=raster_col-2*filtretype;
fwrite(&buff, 4, 1, fich_out_red); //image width
buff=raster_line-2*filtretype;
fwrite(&buff, 4, 1, fich_out_red); //image height
buff=1;
fwrite(&buff, 2, 1, fich_out_red); //plans (always 1)
buff=24;
fwrite(&buff, 2, 1, fich_out_red); //depth (always 24 bits)
buff=0;
fwrite(&buff, 4, 1, fich_out_red);  //compression (without)
buff=0;
fwrite(&buff, 4, 1, fich_out_red);  //image size
buff=0;
fwrite(&buff, 4, 1, fich_out_red);  //horizontal resolution 0
buff=0;
fwrite(&buff, 4, 1, fich_out_red);  //vertical resolution 0
buff=0;
fwrite(&buff, 4, 1, fich_out_red);  //number pallet colors 0
buff=0;
fwrite(&buff, 4, 1, fich_out_red);  //large number of colors 0
//


for (i=(filtretype); i<(raster_line-filtretype); i++)
{
fseek(fich_out_red, 54+(raster_line-i-2*filtretype+2)*((raster_col-2*filtretype)*3+k), SEEK_SET);
	for (j=(filtretype); j<(raster_col-filtretype); j++)
	{
		if (((denom[i*raster_col+j])>=seuildenom)*((lambdatemp[i*raster_col+j])>0)) 
		{
		fprintf(fich_out_num, "%d\t", num[i*raster_col+j]);
		fprintf(fich_out_denom, "%d\t", denom[i*raster_col+j]);
		fprintf(fich_out_rap, "%lf\t", rapporttemp[i*raster_col+j]);
		fprintf(fich_out_pva, "%.9lf\t", pvaluetemp[i*raster_col+j]);
		fprintf(fich_out_lam, "%.1f\t", lambdatemp[i*raster_col+j]);
		
		if (two_tailed) {
		if (((pvaluetemp[i*raster_col+j])>(tresh/(2*nbpx)))*((pvaluetemp[i*raster_col+j])>(0.025/nbpx))*((pvaluetemp[i*raster_col+j])<(1-(tresh/(2*nbpx))))*((pvaluetemp[i*raster_col+j])<(1-(0.025/nbpx))))
			{
			//not labeled 2t
			buff=(int)ceil(256*(float)(denom[i*raster_col+j]-mindenom)/maxdenom);
			fwrite(&buff, 1, 1, fich_out_red);
			fwrite(&buff, 1, 1, fich_out_red);
			fwrite(&buff, 1, 1, fich_out_red);
			fprintf(fich_out_rap_pos, "NaN\t");
			fprintf(fich_out_rap_first, "NaN\t");
			fprintf(fich_out_num_pos, "NaN\t");
			fprintf(fich_out_num_first, "NaN\t");
			fprintf(fich_out_denom_pos, "NaN\t");
			fprintf(fich_out_denom_first, "NaN\t");
			}
		if (((pvaluetemp[i*raster_col+j])<=(tresh/(2*nbpx)))*((pvaluetemp[i*raster_col+j])>(0.025/nbpx)))
			{
			//labeled at the upper tresh threshold
			buff=0;
			fwrite(&buff, 1, 1, fich_out_red);
			
			fwrite(&buff, 1, 1, fich_out_red);
			buff=128;
			fwrite(&buff, 1, 1, fich_out_red);
			nb_sup_seuil++;
			fprintf(fich_out_fin, "Pixel i=%d j=%d fraction=%lf lambda=%.1f norme=%d denom=%d num=%d pvalue=%.9lf positive (above ref) to probability threshold=%f second class 2-tailed\n", i, j, rapporttemp[i*raster_col+j], lambdatemp[i*raster_col+j], normasse[i*raster_col+j], denom[i*raster_col+j], num[i*raster_col+j], pvaluetemp[i*raster_col+j], tresh/(2*nbpx));
			fprintf(fich_out_rap_pos, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_rap_first, "NaN\t");
			fprintf(fich_out_num_pos, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_num_first, "NaN\t");
			fprintf(fich_out_denom_pos, "%d\t", denom[i*raster_col+j]);
			fprintf(fich_out_denom_first, "NaN\t");
			}
			
		if (((pvaluetemp[i*raster_col+j])>=(1-(tresh/(2*nbpx))))*((pvaluetemp[i*raster_col+j])<(1-(0.025/nbpx))))
			{
			//labeled at the lower tresh threshold
			buff=128;
			fwrite(&buff, 1, 1, fich_out_red);
			buff=0;
			fwrite(&buff, 1, 1, fich_out_red);
			
			fwrite(&buff, 1, 1, fich_out_red);
			nb_inf_seuil++;
			fprintf(fich_out_fin, "Pixel i=%d j=%d fraction=%lf lambda=%.1f norme=%d denom=%d num=%d pvalue=%.9lf positive (below ref) to probability threshold=%f second class 2-tailed\n", i, j, rapporttemp[i*raster_col+j], lambdatemp[i*raster_col+j], normasse[i*raster_col+j], denom[i*raster_col+j], num[i*raster_col+j], pvaluetemp[i*raster_col+j], tresh/(2*nbpx));
			fprintf(fich_out_rap_pos, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_rap_first, "NaN\t");
			fprintf(fich_out_num_pos, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_num_first, "NaN\t");
			fprintf(fich_out_denom_pos, "%d\t", denom[i*raster_col+j]);
			fprintf(fich_out_denom_first, "NaN\t");
			}
		if ((pvaluetemp[i*raster_col+j])<=(0.025/nbpx)) 
			{
			//labeled at the upper bonferroni threshold
			buff=0;
			fwrite(&buff, 1, 1, fich_out_red);
			
			fwrite(&buff, 1, 1, fich_out_red);
			buff=255;
			fwrite(&buff, 1, 1, fich_out_red);
			nb_sup_bonf++;
			fprintf(fich_out_fin, "Pixel i=%d j=%d fraction=%lf lambda=%.1f norme=%d denom=%d num=%d pvalue=%.9lf positive (above ref) Bonferroni=%f first class 2-tailed\n", i, j, rapporttemp[i*raster_col+j], lambdatemp[i*raster_col+j], normasse[i*raster_col+j], denom[i*raster_col+j], num[i*raster_col+j], pvaluetemp[i*raster_col+j], (0.025/nbpx));

			fprintf(fich_out_rap_pos, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_rap_first, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_num_pos, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_num_first, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_denom_pos, "%d\t", denom[i*raster_col+j]);
			fprintf(fich_out_denom_first, "%d\t", denom[i*raster_col+j]);
			}
		if ((pvaluetemp[i*raster_col+j])>=(1-(0.025/nbpx)))
			{
			//labeled at the lower bonferroni threshold
			
			buff=255;
			fwrite(&buff, 1, 1, fich_out_red);
			buff=0;
			fwrite(&buff, 1, 1, fich_out_red);
			
			fwrite(&buff, 1, 1, fich_out_red);
			nb_inf_bonf++;
			fprintf(fich_out_fin, "Pixel i=%d j=%d fraction=%lf lambda=%.1f norme=%d denom=%d num=%d pvalue=%.9lf positive (below ref) Bonferroni=%f first class 2-tailed\n", i, j, rapporttemp[i*raster_col+j], lambdatemp[i*raster_col+j], normasse[i*raster_col+j], denom[i*raster_col+j], num[i*raster_col+j], pvaluetemp[i*raster_col+j], (0.025/nbpx));

			fprintf(fich_out_rap_pos, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_rap_first, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_num_pos, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_num_first, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_denom_pos, "%d\t", denom[i*raster_col+j]);
			fprintf(fich_out_denom_first, "%d\t", denom[i*raster_col+j]);
			}

		
			
		}
		
		else if (!(two_tailed)) {
		
		if (((pvaluetemp[i*raster_col+j])>(tresh/nbpx))*((pvaluetemp[i*raster_col+j])>(0.05/nbpx)))
			{
			//not labeled 1t
			buff=(int)ceil(256*(float)(denom[i*raster_col+j]-mindenom)/maxdenom);
			fwrite(&buff, 1, 1, fich_out_red);
			fwrite(&buff, 1, 1, fich_out_red);
			fwrite(&buff, 1, 1, fich_out_red);
			fprintf(fich_out_rap_pos, "NaN\t");
			fprintf(fich_out_rap_first, "NaN\t");
			fprintf(fich_out_num_pos, "NaN\t");
			fprintf(fich_out_num_first, "NaN\t");
			fprintf(fich_out_denom_pos, "NaN\t");
			fprintf(fich_out_denom_first, "NaN\t");
			}
		if (((pvaluetemp[i*raster_col+j])<=(tresh/nbpx))*((pvaluetemp[i*raster_col+j])>(0.05/nbpx)))
			{
			//labeled 1t 2nd class 
			buff=0;
			fwrite(&buff, 1, 1, fich_out_red);
			fwrite(&buff, 1, 1, fich_out_red);
			buff=255;
			fwrite(&buff, 1, 1, fich_out_red);
			nbrouge++;
			fprintf(fich_out_fin, "Pixel i=%d j=%d fraction=%lf lambda=%.1f norme=%d denom=%d num=%d pvalue=%.9lf positive to probality threshold=%f second class\n", i, j, rapporttemp[i*raster_col+j], lambdatemp[i*raster_col+j], normasse[i*raster_col+j], denom[i*raster_col+j], num[i*raster_col+j], pvaluetemp[i*raster_col+j], tresh/nbpx);

			fprintf(fich_out_rap_pos, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_rap_first, "NaN\t");
			fprintf(fich_out_num_pos, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_num_first, "NaN\t");
			fprintf(fich_out_denom_pos, "%d\t", denom[i*raster_col+j]);
			fprintf(fich_out_denom_first, "NaN\t");

			}
		if (((pvaluetemp[i*raster_col+j])<=(0.05/nbpx)))
			{
			//labeled 1t 1st class
			buff=255;
			fwrite(&buff, 1, 1, fich_out_red);
			buff=0;
			fwrite(&buff, 1, 1, fich_out_red);
			fwrite(&buff, 1, 1, fich_out_red);
			nbbleu++;
			fprintf(fich_out_fin, "Pixel i=%d j=%d fraction=%lf lambda=%.1f norme=%d denom=%d num=%d pvalue=%.9lf positive to Bonferroni probability=%f first class\n", i, j, rapporttemp[i*raster_col+j], lambdatemp[i*raster_col+j], normasse[i*raster_col+j], denom[i*raster_col+j], num[i*raster_col+j], pvaluetemp[i*raster_col+j], (0.05/nbpx));

			fprintf(fich_out_rap_pos, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_rap_first, "%lf\t", rapporttemp[i*raster_col+j]);
			fprintf(fich_out_num_pos, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_num_first, "%d\t", num[i*raster_col+j]);
			fprintf(fich_out_denom_pos, "%d\t", denom[i*raster_col+j]);
			fprintf(fich_out_denom_first, "%d\t", denom[i*raster_col+j]);
			}
		
		}
		
		
		
				
		
		}
		else
		{
		fprintf(fich_out_num, "NaN\t");
		fprintf(fich_out_denom, "NaN\t");
		
		fprintf(fich_out_rap, "NaN\t");
		fprintf(fich_out_pva, "NaN\t");
		fprintf(fich_out_lam, "NaN\t");
		fprintf(fich_out_rap_pos, "NaN\t");
		fprintf(fich_out_rap_first, "NaN\t");
		fprintf(fich_out_num_pos, "NaN\t");
		fprintf(fich_out_num_first, "NaN\t");
		fprintf(fich_out_denom_pos, "NaN\t");
		fprintf(fich_out_denom_first, "NaN\t");
		buff=0;
		fwrite(&buff, 1, 1, fich_out_red);
		fwrite(&buff, 1, 1, fich_out_red);
		fwrite(&buff, 1, 1, fich_out_red);
		
		}
		

	}
		fprintf(fich_out_num, "\n");
		fprintf(fich_out_denom, "\n");
		
		fprintf(fich_out_rap, "\n");
		fprintf(fich_out_pva, "\n");
		fprintf(fich_out_lam, "\n");
		
		fprintf(fich_out_rap_pos, "\n");
		fprintf(fich_out_rap_first, "\n");
		fprintf(fich_out_num_pos, "\n");
		fprintf(fich_out_num_first, "\n");
		fprintf(fich_out_denom_pos, "\n");
		fprintf(fich_out_denom_first, "\n");
		
	for (l=0; l<k; l++)
	{
		buff=0;
		fwrite(&buff, 1, 1, fich_out_red);
	}

}

if (two_tailed) {
fprintf(fich_out_fin, "Number of 2nd class pixels below the reference (dark blue) = %d\n", nb_inf_seuil);
fprintf(fich_out_fin, "Number of 1st class pixels below the reference (light blue) = %d\n", nb_inf_bonf);
fprintf(fich_out_fin, "Number of 2nd class pixels above the reference (dark red) = %d\n", nb_sup_seuil);
fprintf(fich_out_fin, "Number de 1st class pixels above the reference (light red) = %d\n", nb_sup_bonf);
}

else {
fprintf(fich_out_fin, "Number of 2nd class pixels (red) = %d\n", nbrouge);
fprintf(fich_out_fin, "Nomber of 1st class pixels (blue) = %d\n", nbbleu);
}

temps_fin=time(NULL);
duree=difftime(temps_fin, temps_debut);
fprintf(fich_out_fin, "Total elapse time (seconds) :%f\n", duree);

fclose(fich_out_fin);
fclose(fich_out_rap);
fclose(fich_out_red);
fclose(fich_out_lam);

fclose(fich_out_num);
fclose(fich_out_denom);


fclose(fich_out_rap_pos);
fclose(fich_out_rap_first);
fclose(fich_out_num_pos);
fclose(fich_out_num_first);
fclose(fich_out_denom_pos);
fclose(fich_out_denom_first);

}


/*free table memory*/
free(rapporttemp);
free(pvaluetemp);
free(num);
free(denom);
free(lambda);
free(lambdatemp);
free(pvalue);
free(normasse);
free(rapport);


/*wait for the memory to be free by all processes and close*/
  MPI_Barrier(MPI_COMM_WORLD);
  MPI_Finalize();
  return(0);
}