﻿/*
SIED_maker v2.32

Copyright (C) 2015
Anthony Delaune
Loïc Thibaud
Laboratoire GPM UMR CNRS 6634 Equipe NanoCARE
Université de Rouen
anthony.delaune@univ-rouen.fr


    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

graphic software for preparing the SIMS-SIED analysis. Open the cameca .im file (32bit Windows format). 

v1.1
correction of the name of the created folders
correction of the name of the masses with spaces in them

v1.2
added the original file name in config.ini
added the name of the original file name of the image.txt

v2.0
editing of 2 configurations simultaneously

v2.1
script editing

v2.2
sum of planes with adjustment : creates a new image plane containing the sum
calculation with progress bar
bug correction : prevents division by zero for displays of a null denominator image 

v2.21
07 jan 2015
compliance with SIED 1.7 requiring raster_line and raster_col to be written in config.ini
added "two-tailed" parameter (before last of config.ini)0 : one-tailed 1 : two-tailed

v2.22
28 jan 2015
added a character \0 at the end of the prefix line (avoid bug)

v2.23
11 feb 2015
english translation 
added a parent directory line in the script : for compatibility

v2.24
23 march 2015
test of a new image adjustment method : x and y projection of images and vector adjustment

v2.30
6 may 2015
addition of 2 images adjustment methods

v2.31
4 feb 2016
recalibration debugging


v2.32
13 december 2017
opening .im files generated by the windows station 32 bit
TODO : opening a config.ini ; the alignment of the planes seems to bug : the results are not as expected 
*/

#include <errno.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

GtkWidget *pwindow;
GtkWidget *progress;
GtkWidget *boitealt = NULL;
GtkWidget *paramalt = NULL;
GtkWidget *paramsum = NULL;
GtkWidget *checkalt = NULL;
GtkWidget *radio2t1 = NULL;
GtkWidget *radio2t2 = NULL;
GtkWidget *checksum = NULL;
GtkWidget *diralt1 = NULL;
GtkWidget *diralt2 = NULL;

GtkWidget *choixplan = NULL;
GtkWidget *choixnum = NULL;
GtkWidget *choixref = NULL;
GtkWidget *choixdenom = NULL;
GtkWidget *choixnorme = NULL;
GtkWidget *checkmoyenne = NULL;
GtkWidget *pVBoxmain;
GdkPixbuf *pixbuf;
GtkWidget *pImage;
GtkWidget *choix_seuil_denom = NULL;
GtkWidget *choix_seuil_moy = NULL;
GtkObject *adjust1 = NULL;
GtkObject *adjust2 = NULL;
GdkBitmap *bitmap_denom = NULL;
GdkBitmap *bitmap_moy = NULL;
GtkWidget *boitemoy = NULL;
GtkWidget *boitenomfich = NULL;
GtkWidget *seuil_proba = NULL;
GtkWidget *radio1, *radio2, *radio3, *radio4, *radio5;
GtkWidget *pText_view; 
char *sPath= NULL;
char *nom_fich_origin= NULL;

unsigned long int maxpix=0;
int nbmass, nbplan, raster_col, raster_lin;
unsigned int *image; //pointer storing all images. It will be necessary to allocate memory 
unsigned int *image_sum;
char nom_mass[5][8]= {}; //mass names
int plan_actif;
unsigned int seuil_denom, seuil_moy;
unsigned int is_sum_calculated=0;
guchar *impix = NULL;
int filtretype=3, two_tailed=0;
int recal_meth=1;


void opensims(gchar *fichier)
{
    int longueur, u, v;
    char *nom_temp;
    FILE *fp;

    int header_size=0,  i=0, j=0, k=0, l=0;
    short a=0, b=0;

    int offset=0;
    long int pixel=0;
    char temp[32];
//we erase the old list of plans
    for (j=0; j<nbplan+1; j++)
    {
        gtk_combo_box_remove_text(GTK_COMBO_BOX(choixplan), 0);
    }
    for (j=0; j<nbmass; j++)
    {
        gtk_combo_box_remove_text (GTK_COMBO_BOX(choixnum), 0);
        gtk_combo_box_remove_text (GTK_COMBO_BOX(choixdenom), 0);
        gtk_combo_box_remove_text (GTK_COMBO_BOX(choixnorme), 0);
        gtk_combo_box_remove_text (GTK_COMBO_BOX(choixref), 0);
    }


    fp=fopen(fichier, "rb");
    fflush(fp);
//we read the number of mass

    fseek(fp, 408, SEEK_SET); //win32bit format (411 for sun16bit)
    fread(&nbmass, 1, 1, fp);
//reading mass names 
    for (i=0; i<nbmass; i++)
    {
        fseek(fp, 717+i*192, SEEK_SET);//win32bit format
        fread(&nom_mass[i], 7, 1, fp);
    }
//mass name correction
    for (j=0; j<nbmass; j++)
    {

        for (i=0; i<7; i++)
        {
            if (nom_mass[j][i]==32)
            {
                nom_mass[j][i]=95;
            }
        }
    }

    fseek(fp, 8, SEEK_SET);//win32bit format
    fread(&header_size, 2, 1, fp);//win32bit format


//number of plans
    fseek(fp, header_size-70, SEEK_SET);//win32bit format
    fread(&nbplan, 1, 1, fp);

//writing the name of the masses and the plan number on each plan
    for (j=0; j<nbplan; j++)
    {
//
        sprintf(temp, "plane %d", (j+1));
        gtk_combo_box_append_text(GTK_COMBO_BOX(choixplan), temp);
    }
        sprintf(temp, "plane_sum");
        gtk_combo_box_append_text(GTK_COMBO_BOX(choixplan), temp);
//research of raster x and y
    fseek(fp, header_size-78, SEEK_SET);
    fread(&a, 1, 1, fp);
    fread(&b, 1, 1, fp);
    raster_col=a + 256*b;

    printf("raster_col: %d\n", raster_col);
    fseek(fp, header_size-76, SEEK_SET);
    fread(&a, 1, 1, fp);
    fread(&b, 1, 1, fp);
    raster_lin=a + 256*b;;

    printf("raster_lin: %d\n", raster_lin);

//memory allocation for images
    image = (unsigned int*) calloc((nbplan+1)*nbmass*raster_col*raster_lin, sizeof(unsigned int));

//reading images and filling tables
    fflush(fp);
    fseek(fp, header_size, SEEK_SET);

    for (i=0; i<nbplan; i++)
    {
        for (j=0; j<nbmass; j++)
        {

            for (k=0; k<raster_lin; k++)
            {
                for (l=0; l<raster_col; l++)
                {

                    fseek(fp, header_size+offset, SEEK_SET);
                    fread(&pixel, 2, 1, fp);//win32bit format

                    offset=offset+4;//win32bit format

                    image[i*nbmass*raster_lin*raster_col+j*raster_col*raster_lin+k*raster_col+l]=pixel;
                }
            }
        }
    }
    fclose(fp); 
    longueur=strlen(fichier);

    for (u=(longueur-3); ((fichier[u])!='/'); u--)
    {
//to remove the .im extension of file
    }
    u++;
    nom_fich_origin = (char*) calloc((longueur-2-u), sizeof(char));
    nom_temp= (char*) calloc(4, sizeof(char));

    for (v=0; v<4; v++)
    {
        nom_temp[v]=fichier[v+u];
    }


    for (v=0; v<(longueur-3-u); v++)
    {
        nom_fich_origin[v]=fichier[v+u];
    }
     nom_fich_origin[v]='\0';
    gtk_entry_set_text(GTK_ENTRY(boitenomfich), nom_fich_origin);
    free(nom_fich_origin);
    free(nom_temp);
    return;
}
void openini(gchar *fichier)
{
    /*  To be written in order to re open an already analyzed image
    */
    return;
}

static void change_seuil_denom(GtkWidget *wid, GtkWidget *win)
{
    int i,j;
    int coord;
    int num_image;
    seuil_denom=gtk_adjustment_get_value(GTK_ADJUSTMENT(adjust1));
    num_image=gtk_combo_box_get_active(GTK_COMBO_BOX(choixdenom));
    if ((seuil_denom!=-1)&&(num_image!=-1))
    {
        for (i=0; i<raster_lin; i++)
        {
            for (j=0; j<raster_col; j++)
            {
                if (image[plan_actif*raster_col*raster_lin*nbmass+num_image*raster_col*raster_lin+i*raster_col+j]<seuil_denom)
                {
                    coord=(i*raster_col+j)*3;
                    impix[coord]=255;
                }
                else
                {
                    coord=(i*raster_col+j)*3;
                    impix[coord]=image[plan_actif*raster_col*raster_lin*nbmass+num_image*raster_col*raster_lin+i*raster_col+j]*255/maxpix;
                }
            }
        }

        gtk_image_set_from_pixbuf(GTK_IMAGE(pImage), pixbuf);
        gtk_widget_show_all(pwindow);
    }
//we get the value of the hscale
    return;
}
static void change_seuil_moy(GtkWidget *wid, GtkWidget *win)
{
//we get the value of the hscale
    int i,j;
    int coord;
    int num_image;
    seuil_moy=gtk_adjustment_get_value(GTK_ADJUSTMENT(adjust2));
    num_image=gtk_combo_box_get_active(GTK_COMBO_BOX(choixdenom));
    if ((seuil_moy!=-1)&&(num_image!=-1))
    {
        for (i=0; i<raster_lin; i++)
        {
            for (j=0; j<raster_col; j++)
            {
                if (image[plan_actif*raster_col*raster_lin*nbmass+num_image*raster_col*raster_lin+i*raster_col+j]<seuil_moy)
                {
                    coord=(i*raster_col+j)*3;
                    impix[coord+1]=255;
                }
                else
                {
                    coord=(i*raster_col+j)*3;
                    impix[coord+1]=image[plan_actif*raster_col*raster_lin*nbmass+num_image*raster_col*raster_lin+i*raster_col+j]*255/maxpix;
                }
            }
        }
        gtk_image_set_from_pixbuf(GTK_IMAGE(pImage), pixbuf);
        gtk_widget_show_all(pwindow);
    }
    return;
}

static void change_plan (GtkWidget *wid, GtkWidget *win)
{
    int i, j;
    plan_actif = gtk_combo_box_get_active(GTK_COMBO_BOX(choixplan));

    for (i=0; i<nbmass; i++)
    {
        gtk_combo_box_remove_text (GTK_COMBO_BOX(choixnum), 0);
        gtk_combo_box_remove_text (GTK_COMBO_BOX(choixdenom), 0);
        gtk_combo_box_remove_text (GTK_COMBO_BOX(choixnorme), 0);
        gtk_combo_box_remove_text (GTK_COMBO_BOX(choixref), 0);
    }

    for (j=0; j<nbmass; j++)
    {
        //filling combobox images
        gtk_combo_box_append_text(GTK_COMBO_BOX(choixnum), nom_mass[j]);
        gtk_combo_box_append_text(GTK_COMBO_BOX(choixdenom), nom_mass[j]);
        gtk_combo_box_append_text(GTK_COMBO_BOX(choixnorme), nom_mass[j]);
        gtk_combo_box_append_text(GTK_COMBO_BOX(choixref), nom_mass[j]);
        gtk_spin_button_set_range(paramsum, 0, 6*nbplan);
        gtk_spin_button_set_value(paramsum, 6*nbplan);

    }

    return;
}

static void change_denom (GtkWidget *wid, GtkWidget *win)
{

    
    int i,j,cpt=0;
    unsigned char tempo;
    char nom_window[32];
    int num_image;
    impix=(guchar*) calloc(raster_col*raster_lin*3, sizeof(guchar));
    num_image=gtk_combo_box_get_active(GTK_COMBO_BOX(choixdenom));
    if (num_image!=-1)
    {
        cpt=0;
        maxpix=0;
        sprintf(nom_window, "plane %d %s", plan_actif+1 ,nom_mass[num_image]);
        gtk_window_set_title(GTK_WINDOW(pwindow), nom_window);
        gtk_window_resize(GTK_WINDOW(pwindow),raster_col+4, raster_lin+4);
//linear scale 
        for (i=0; i<raster_lin; i++)
        {
            for (j=0; j<raster_col; j++)
            {
                if ((image[plan_actif*raster_col*raster_lin*nbmass+num_image*raster_col*raster_lin+i*raster_col+j])>maxpix)
                    maxpix=image[plan_actif*raster_col*raster_lin*nbmass+num_image*raster_col*raster_lin+i*raster_col+j];
            }
        }
//pixbuf filling
                if (maxpix==0)
                {
                maxpix=1;
                }
        for (i=0; i<raster_lin; i++)
        {
            for (j=0; j<raster_col; j++)
            {
                tempo=(image[plan_actif*raster_col*raster_lin*nbmass+num_image*raster_col*raster_lin+i*raster_col+j])*255/maxpix;
                impix[cpt]=tempo;
                impix[cpt+1]=tempo;
                impix[cpt+2]=tempo;
                cpt=cpt+3;
            }
        }
        pixbuf= gdk_pixbuf_new_from_data(impix, GDK_COLORSPACE_RGB, FALSE, 8, raster_col, raster_lin, raster_col*3, NULL, NULL);
        gtk_image_set_from_pixbuf(GTK_IMAGE(pImage), pixbuf);
        gtk_widget_show_all(pwindow);
        gtk_adjustment_set_value (GTK_ADJUSTMENT(adjust1), 1);
        gtk_adjustment_set_value (GTK_ADJUSTMENT(adjust2), 0);
        gtk_adjustment_set_upper (GTK_ADJUSTMENT(adjust1), maxpix+50);
        gtk_adjustment_set_upper (GTK_ADJUSTMENT(adjust2), maxpix+50);
// to adjust the range
    }
    return;
}

static void open_im (GtkWidget *wid, GtkWidget *win)
{

    GtkWidget *pFileSelection;
    GtkWidget *pParent;
    gchar *sChemin;
    char *extension;
    int  length;
    extension = (char*) calloc(3, sizeof(char));
    pParent = GTK_WIDGET(win);
    /* creating the selection window*/
    pFileSelection = gtk_file_chooser_dialog_new("Open...",
                     GTK_WINDOW(pParent),
                     GTK_FILE_CHOOSER_ACTION_OPEN,
                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                     GTK_STOCK_OPEN, GTK_RESPONSE_OK,
                     NULL);
    /*we limit the actions to this window*/
    gtk_window_set_modal(GTK_WINDOW(pFileSelection), TRUE);
    /*window display*/
    switch(gtk_dialog_run(GTK_DIALOG(pFileSelection)))
    {
    case GTK_RESPONSE_OK:
        /*determine path*/
        sChemin = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pFileSelection));
        length=strlen(sChemin);
        free(sPath);
        sPath = NULL;
        sPath = (char*) calloc(length-3, sizeof(char));
        strncpy(sPath, sChemin, length-3);
        extension[0]=sChemin[length-3];
        extension[1]=sChemin[length-2];
        extension[2]=sChemin[length-1];
        printf("path=%s\nfile=%s\nextension=%s\n", sPath, sChemin, extension);
        if (!strcmp(extension,".im"))
        {
            free(image); //free memory 
            image= NULL; //reinitialization
            printf("Reading SIMS file\n");
            opensims(sChemin);
            printf("OK\n");
        }
        if (!strcmp(extension,"ini"))
        {
            openini(sChemin);
        }
        g_free(sChemin);
        free(extension);
        break;
    default:
        break;
    }
    gtk_widget_destroy(pFileSelection);
    return;
}

static void clicmoy (GtkWidget *wid, GtkWidget *win)
{
    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)))
    {
        gtk_entry_set_text(GTK_ENTRY(boitemoy), "-0.1");
        gtk_widget_show(choix_seuil_moy);
        gtk_widget_hide(boitemoy);
    }
    else
    {
        gtk_entry_set_text(GTK_ENTRY(boitemoy), "0.0037");
        gtk_widget_hide(choix_seuil_moy);
        gtk_widget_show(boitemoy);
    }
    return;
}

static void radio2t_set_1 (GtkWidget *wid, GtkWidget *win)
{
    two_tailed = 0;
    return;
}
static void radio2t_set_2 (GtkWidget *wid, GtkWidget *win)
{
    two_tailed = 1;
    return;
}

static void radio_set_1 (GtkWidget *wid, GtkWidget *win)
{
    filtretype = 0;
    return;
}
static void radio_set_2 (GtkWidget *wid, GtkWidget *win)
{
    filtretype = 1;
    return;
}
static void radio_set_3 (GtkWidget *wid, GtkWidget *win)
{
    filtretype = 2;
    return;
}
static void radio_set_4 (GtkWidget *wid, GtkWidget *win)
{
    filtretype=3;
    return;
}
static void radio_set_5 (GtkWidget *wid, GtkWidget *win)
{
    filtretype=4;
    return;
}
static void changeproba (GtkWidget *wid, GtkWidget *win)
{
    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)))
    {
        gtk_entry_set_text(GTK_ENTRY(seuil_proba), "0.1");
        gtk_widget_hide(seuil_proba);
    }
    else
    {
        gtk_widget_show(seuil_proba);
    }
    return;
}
static void clicalt (GtkWidget *wid, GtkWidget *win)
{
    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)))
    {
        gtk_widget_hide(boitealt);
        gtk_widget_hide(paramalt);
        gtk_widget_hide(diralt1);
        gtk_widget_hide(diralt2);
    }
    else
    {
        gtk_widget_show(boitealt);
        gtk_widget_show(diralt1);
        gtk_widget_show(diralt2);
        gtk_widget_show(paramalt);
    }
    return;
}

double **createTable(int nbLin, int nbCol)
//table declaration for the joint histogram (6 may 2015)
{

	double **tableau = (double **)malloc(sizeof(double*)*nbLin);
	double *tableau2 = (double *)malloc(sizeof(double)*nbCol*nbLin);
	int i;
	for( i = 0 ; i < nbLin ; i++){
		tableau[i] = &tableau2[i*nbCol];
	}
	return tableau;
}

void freeTable(double **tableau){
	free(tableau[0]);
	free(tableau);
}

static void clicsum_histogramme (void)
{
    //readjustment by joint histogram method (6 may 2015)
    //Loïc Thibaud and Anthony Delaune
    printf("Joint Entropy method");
    int id_im_ref;
    int max_shift;
    int i, j, k, l, m, n, num_plan, delta_i, delta_j, nb_pix, delta_i_opt, delta_j_opt;
    int *im_norm_ref, *im_norm_act;
    unsigned int max_im_ref, max_im_active, min_im_ref, min_im_active;
    int *shift_i;
    int *shift_j;
    double **H;

    double  tempo;

    unsigned int temp_pix;
    float hist_2D[256][256]={{0}};
    FILE *file_out;
    id_im_ref= gtk_combo_box_get_active(GTK_COMBO_BOX(choixref));
    gtk_grab_add(progress);

    if (nbplan<2 || id_im_ref==-1 )
    {
        
        printf("Incomplete selection or only one plane. Nothing will be calculated\n");
    }
    else
    {
        if (is_sum_calculated==1)
        {
        printf("The sum is already calculated. Nothing will be calculated\n");
        }
        else {
        //the sum is calculated
        max_shift = gtk_spin_button_get_value_as_int(paramsum);

shift_i = (int*) calloc(nbplan, sizeof(int));
shift_j = (int*) calloc(nbplan, sizeof(int));
H=createTable((2*max_shift+1), (2*max_shift+1));
im_norm_ref = (int*) calloc(raster_col*raster_lin, sizeof(int));

shift_i[0]=0;
shift_j[0]=0;
min_im_ref=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+0+0*raster_col];
max_im_ref=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+0+0*raster_col];
///search in the image file 1 for minimum and maximum values
    for (j=0; j<raster_lin; j++)
    {
        for (i=0; i<raster_col; i++)
        {
            if ((image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col])<min_im_ref)
            {
                min_im_ref=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col];
            }

            if ((image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col])>max_im_ref)
            {
                max_im_ref=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col];
            }

        }
    }

    for (i=0; i<(raster_lin); i++)
    {
        for (j=0; j<raster_col; j++)
        {
            im_norm_ref[i*raster_col+j]=(int)((float)image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+j+i*raster_col]-(float)min_im_ref)/((float)max_im_ref-(float)min_im_ref)*255;
        }
    }



///---------------------------------------------------------------------------------------main loop

    for (num_plan = 2 ; num_plan < (nbplan+1) ; num_plan++)
    {
    ///search in the image file 2 for minimum and maximum values
gtk_progress_bar_set_fraction(progress, (float)((float)(num_plan-1)/((float)(nbplan+1))));
    gtk_main_iteration();

        im_norm_act = (int*)calloc(raster_col*raster_lin, sizeof(int)) ; 
min_im_active=image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+0+0*raster_col];
max_im_active=image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+0+0*raster_col];

        for (j=0; j<raster_lin; j++)
        {
            for (i=0; i<raster_col; i++)
            {
                if (image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col]<min_im_active)
                {
                    min_im_active=image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col];//val_image_out_2b = image
                }

                if (image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col]>max_im_active)
                {
                    max_im_active=image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col];//val_image_out_2b = image
                }

            }
        }

        //normalize image number 2 to grayscale

        for (i=0; i<(raster_lin); i++)
        {
            for (j=0; j<raster_col; j++)
            {
                im_norm_act[i*raster_col+j]=(int)((float)image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i*raster_col+j]-(float)min_im_active)/((float)max_im_active-(float)min_im_active)*255; //im_norm_act
            }
        }

    ///calculation of the joint histogram of the two images
        for (delta_j= (-max_shift); delta_j<(max_shift+1); delta_j++)
        {
            for (delta_i= (-max_shift); delta_i<(max_shift+1); delta_i++)
            {
                nb_pix = 0;

                for (j=0 ; j<raster_lin; j++)
                {
                    for (i=0 ; i<raster_col; i++)
                    {

                        if (((i-delta_i)>-1) && ((j-delta_j)>-1) && ((i-delta_i)<raster_col) && ((j-delta_j)<raster_lin))
                        {
                            m = im_norm_ref[i+j*raster_col];
                            n = im_norm_act[(i-delta_i)+(j-delta_j)*raster_col];
                            hist_2D[m][n]=1+hist_2D[m][n];
                            nb_pix++;


                        }

                    }// i loop out

                }

            //calculation of H[delta_i,delta_j]
                for (m=0; m<256; m++)
                {
                    for (n=0; n<256; n++)
                    {

                        if ((hist_2D[m][n]>0) && (nb_pix!=0))
                        {
                            H[delta_i+max_shift][delta_j+max_shift] += ((-hist_2D[m][n])/nb_pix)*log10(((hist_2D[m][n])/nb_pix));
                        }
                        hist_2D[m][n]=0;


                    }
                }
            }//delta_i loop out
        }//delta_j loop out



        tempo=H[0][0];
        delta_i_opt=0;
        delta_j_opt=0;

        for (delta_j=(-max_shift); delta_j<(max_shift+1); delta_j++)
        {

           for (delta_i=(-max_shift); delta_i<(max_shift+1); delta_i++)
           {
                if ((H[delta_i+max_shift][delta_j+max_shift])<tempo)
                {
                    tempo=H[delta_i+max_shift][delta_j+max_shift];
                    delta_i_opt=delta_i;
                    delta_j_opt=delta_j;


                }
                H[delta_i+max_shift][delta_j+max_shift] = 0;

           }
       }


    shift_i[num_plan-1]=delta_i_opt; 
    shift_j[num_plan-1]=delta_j_opt;
    free(im_norm_act);
    gtk_progress_bar_set_fraction(progress, (float)((float)num_plan/((float)(nbplan+1))));
    gtk_main_iteration();

    }///end of main loop

    free(im_norm_ref);
    freeTable(H);

//sum of plans
   for (n=0; n<nbmass; n++)
   {
              
        for (i=0; i<raster_lin; i++)
        {
            for (j=0; j<raster_col; j++)
            {
                temp_pix=0;
                
                for (m=0; m<nbplan; m++)
                    {
                    if (((i-shift_j[m])<0) || ((i-shift_j[m])>(raster_lin-1)) || ((j-shift_i[m])<0) || ((j-shift_i[m])>(raster_col-1)))
                    {
                    // nothing !
                    }
                    else {
                    temp_pix+=image[m*raster_col*raster_lin*nbmass+n*raster_col*raster_lin+(i-shift_j[m])*raster_col+j-shift_i[m]];
                    }
                    }
                image[nbplan*raster_col*raster_lin*nbmass+n*raster_col*raster_lin+(i)*raster_col+j]=temp_pix;
            }
        }

   }
file_out=fopen("shift.txt", "w");
for (m=0; m<nbplan; m++)
{
    fprintf(file_out, "%d\t%d\n", shift_i[m], shift_j[m]);
}
	fclose(file_out);


    free(shift_i);
    free(shift_j);
}
}
        gtk_progress_bar_set_fraction(progress, 1);
    gtk_main_iteration();

    gtk_grab_remove(progress);
    return;
}

static void clicsum_SMC (void)
{
    //readjustment by least square method (6 may 2015)
    //Loïc Thibaud and Anthony Delaune
    printf("Least square method");
    int id_im_ref;
    int max_shift;
    int i, j, k, l, m, n, num_plan, delta_i, delta_j, nb_pix, delta_i_opt, delta_j_opt;
    int *im_norm_ref, *im_norm_act;
    int max_im_ref, max_im_active, min_im_ref, min_im_active;
    int *shift_i;
    int *shift_j;
    double **H;
    double temp_diff;
    double crit_min, tempo;
    int k_min, l_min;
    unsigned int temp_pix;


    FILE *file_out;
    id_im_ref= gtk_combo_box_get_active(GTK_COMBO_BOX(choixref));
    gtk_grab_add(progress);

    if (nbplan<2 || id_im_ref==-1 )
    {
        
        printf("Incomplete selection or only one plane. Nothing will be calculated\n");
    }
    else
    {
        if (is_sum_calculated==1)
        {
        printf("The sum is already calculated. Nothing will be calculated\n");
        }
        else {
        //the sum is calculated
        max_shift = gtk_spin_button_get_value_as_int(paramsum);

shift_i = (int*) calloc(nbplan, sizeof(int));
shift_j = (int*) calloc(nbplan, sizeof(int));
H=createTable((2*max_shift+1), (2*max_shift+1));

for (i=0; i<(2*max_shift+1); i++)
    {
        for (j=0; j<(2*max_shift+1); j++)
        {
        H[i][j]=0;
        }
    }


im_norm_ref = (int*) calloc(raster_col*raster_lin, sizeof(int));
im_norm_act = (int*) calloc(raster_col*raster_lin, sizeof(int));

shift_i[0]=0;
shift_j[0]=0;
min_im_ref=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+0+0*raster_col];
max_im_ref=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+0+0*raster_col];
///search in the image file 1 for minimum and maximum values
    for (j=0; j<raster_lin; j++)
    {
        for (i=0; i<raster_col; i++)
        {
            if (image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col]<min_im_ref)//search of minimum 
            {
                min_im_ref=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col];
            }

            if (image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col]>max_im_ref)//search of maxima
            {
                max_im_ref=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col];
            }

        }
    }


///normalize image number 1 to grayscale (256 levels)

    for (j=0; j<(raster_lin); j++)
    {
        for (i=0; i<raster_col; i++)
        {
            im_norm_ref[i*raster_col+j]=(int)( 255*((float)image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col]-(float)min_im_ref)/((float)max_im_ref-(float)min_im_ref));

        }
    }


///----------------------------------------------------------------------main loop

    for (num_plan = 2 ; num_plan < (nbplan+1) ; num_plan++)
    {
    gtk_progress_bar_set_fraction(progress, (float)((float)(num_plan-1)/((float)(nbplan+1))));
    gtk_main_iteration();

    ///search in the image file 2 for minimum and maximum values

        im_norm_act = (int*)calloc(raster_col*raster_lin, sizeof(int)) ; 
min_im_active=image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+0+0*raster_col];
max_im_active=image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+0+0*raster_col];

        for (j=0; j<raster_lin; j++)
        {
            for (i=0; i<raster_col; i++)
            {
                if (image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col]<min_im_active)
                {
                    min_im_active=image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col];
                }

                if (image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col]>max_im_active)
                {
                    max_im_active=image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i+j*raster_col];
                }

            }
        }

    ///normalize image number 1 to grayscale (256 levels)

        //normalize image number 2 to grayscale

        for (i=0; i<(raster_lin); i++)
        {
            for (j=0; j<raster_col; j++)
            {
                im_norm_act[i*raster_col+j]=(int)( 255*((float)image[(num_plan-1)*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+i*raster_col+j]-(float)min_im_active)/((float)max_im_active-(float)min_im_active)); //im_norm_act

            }
        }

    ///calculation of the true-likelyhood function for each offset
        for (delta_j= (-max_shift); delta_j<(max_shift+1); delta_j++)
        {
            for (delta_i= (-max_shift); delta_i<(max_shift+1); delta_i++)
            {
                nb_pix = 0;
                H[delta_i+max_shift][delta_j+max_shift]=0;
                for (j=0 ; j<raster_lin; j++)
                {
                    for (i=0 ; i<raster_col; i++)
                    {

                        if (((i-delta_i)>-1) && ((j-delta_j)>-1) && ((i-delta_i)<raster_col) && ((j-delta_j)<raster_lin))
                        {
                            m = im_norm_ref[i+j*raster_col];
                            n = im_norm_act[(i-delta_i)+(j-delta_j)*raster_col];
                            H[delta_i+max_shift][delta_j+max_shift] +=(double)((m-n)*(m-n));
                             nb_pix++;


                        }

                    }// sortie boucle i

                }
                H[delta_i+max_shift][delta_j+max_shift] = (double)(H[delta_i+max_shift][delta_j+max_shift])/(double)(nb_pix);
                H[delta_i+max_shift][delta_j+max_shift] = (double)(H[delta_i+max_shift][delta_j+max_shift])/(double)(nb_pix);

            }//sortie boucle delta_i
        }//sortie boucle delta_j



        tempo=H[0][0];
        delta_i_opt=0;
        delta_j_opt=0;
        for (delta_j=(-max_shift); delta_j<(max_shift+1); delta_j++)
        {

           for (delta_i=(-max_shift); delta_i<(max_shift+1); delta_i++)
           {

                if ((H[delta_i+max_shift][delta_j+max_shift])<tempo)
                {
                    tempo=H[delta_i+max_shift][delta_j+max_shift];
                    delta_i_opt=delta_i;
                    delta_j_opt=delta_j;
                }



           }
       }


    shift_i[num_plan-1]=delta_i_opt; 
    shift_j[num_plan-1]=delta_j_opt;
    free(im_norm_act);
    gtk_progress_bar_set_fraction(progress, (float)((float)num_plan/((float)(nbplan+1))));
    gtk_main_iteration();

    }///end of main loop

    free(im_norm_ref);
    freeTable(H);

//sum of plans
   for (n=0; n<nbmass; n++)
   {
               
        for (i=0; i<raster_lin; i++)
        {
            for (j=0; j<raster_col; j++)
            {
                temp_pix=0;
                
                for (m=0; m<nbplan; m++)
                    {
                    if (((i-shift_j[m])<0) || ((i-shift_j[m])>(raster_lin-1)) || ((j-shift_i[m])<0) || ((j-shift_i[m])>(raster_col-1)))
                    {
                    // nothing !
                    }
                    else {
                    temp_pix+=image[m*raster_col*raster_lin*nbmass+n*raster_col*raster_lin+(i-shift_j[m])*raster_col+j-shift_i[m]]; //inversion between i and j
                    }
                    }


                image[nbplan*raster_col*raster_lin*nbmass+n*raster_col*raster_lin+(i)*raster_col+j]=temp_pix;
            }
        }

   }
file_out=fopen("shift.txt", "w");
for (m=0; m<nbplan; m++)
{
    fprintf(file_out, "%d\t%d\n", shift_i[m], shift_j[m]);
}
	fclose(file_out);


    free(shift_i);
    free(shift_j);
}
}
    gtk_progress_bar_set_fraction(progress, 1);
    gtk_main_iteration();

    gtk_grab_remove(progress);
    return;
}

static void clicsum_alt (void)
{
    //alternative sum by projection x and y  2.24 (23 march 2015)
     printf("projection method");
    int id_im_ref;
    int max_shift;
    int i, j, k, l, m, n;
    float *vect_x_ref, *vect_y_ref, *vect_x_active, *vect_y_active;
    int max_im_ref, max_im_active;
    int *shift_x;
    int *shift_y;
    double *critere;
    double temp_diff;
    double crit_min;
    int k_min, l_min;
    unsigned int temp_pix;
    FILE *file_out;
    id_im_ref= gtk_combo_box_get_active(GTK_COMBO_BOX(choixref));
    gtk_grab_add(progress);
    if (nbplan<2 || id_im_ref==-1 )
    {
        
        printf("Incomplete selection or only one plane. Nothing will be calculated\n");
    }
    else
    {
        if (is_sum_calculated==1)
        {
        printf("The sum is already calculated. Nothing will be calculated\n");
        }
        else {
        // calculation of the sum
        max_shift = gtk_spin_button_get_value_as_int(paramsum);
shift_x = (int*) calloc(nbplan, sizeof(int));
shift_y = (int*) calloc(nbplan, sizeof(int));
critere = (double*) calloc((2*max_shift+1), sizeof(double));

vect_x_ref = (float*) calloc(raster_col, sizeof(float));
vect_x_active = (float*) calloc(raster_col, sizeof(float));
vect_y_ref = (float*) calloc(raster_lin, sizeof(float));
vect_y_active = (float*) calloc(raster_lin, sizeof(float));

shift_x[0]=0;
shift_y[0]=0;
// calculate the y reference projection
max_im_ref=0;

for (k=0; k<raster_lin; k++)
{
    for (l=0; l<raster_col; l++)
    {
        vect_y_ref[k]+=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+(k)*raster_col+l];
        vect_x_ref[l]+=image[0*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+(k)*raster_col+l];
    }
}


max_im_ref=vect_y_ref[0];
for (k=1; k<raster_lin; k++)
{
max_im_ref+=vect_y_ref[k];
}
max_im_ref=max_im_ref/raster_lin;
for (k=0; k<raster_lin; k++)
{
vect_y_ref[k]=vect_y_ref[k]/max_im_ref;
}

max_im_ref=vect_x_ref[0];
//calculate the x reference projection
for (l=1; l<raster_col; l++)
{
 max_im_ref+=vect_x_ref[l];
}
max_im_ref=max_im_ref/raster_col;

for (l=0; l<raster_col; l++)
{
    vect_x_ref[l]=vect_x_ref[l]/max_im_ref;
}


// determination of shifts for each plan
            for (m=1; m<nbplan; m++)
            {

max_im_active=0;
for (k=0; k<raster_lin; k++)
{
vect_y_active[k]=0;
}

for (l=0; l<raster_col; l++)
{

    vect_x_active[l]=0;
}
//calculation of the active projection vector y

for (k=0; k<raster_lin; k++)
{
    for (l=0; l<raster_col; l++)
    {
        vect_y_active[k]+=image[m*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+(k)*raster_col+l];
        vect_x_active[l]+=image[m*raster_lin*raster_col*nbmass+id_im_ref*raster_col*raster_lin+(k)*raster_col+l];

    }
}
max_im_active=vect_y_active[0];

for (k=1; k<raster_lin; k++)
{
    max_im_active+=vect_y_active[k];

}
max_im_active=max_im_active/raster_lin;
for (k=0; k<raster_lin; k++)
{
vect_y_active[k]=vect_y_active[k]/max_im_active;
}//calculation of the active projection vector x
max_im_active=vect_x_active[0];
for (l=1; l<raster_col; l++)
{
max_im_active+=vect_x_active[l];
}
max_im_active=max_im_active/raster_col;
for (l=0; l<raster_col; l++)
{

    vect_x_active[l]=vect_x_active[l]/max_im_active;
}
//readjustment y
                for (k=-max_shift; k<(max_shift+1); k++)
                {
                    critere[k+max_shift]=0.0;

                    for (i=max_shift; i<(raster_lin-max_shift); i++)
                    {


                        temp_diff=vect_y_active[i-k]-vect_y_ref[i];


                        critere[(k+max_shift)]+=sqrt(temp_diff*temp_diff);
                    }
                    }

                           // for each plan, the weakest criterion is chosen
            crit_min=critere[0];
            k_min=-max_shift;

            for (k=-max_shift; k<(max_shift+1); k++)
            {

                                    if (critere[(k+max_shift)]<crit_min)
                    {
                    crit_min=critere[(k+max_shift)];
                    k_min=k;
                    }
                }

                 shift_x[m]=k_min;

                  gtk_progress_bar_set_fraction(progress, (float)((float)m/((float)2*(nbplan-1))));
                gtk_main_iteration();
//readjustment x
                for (l=-max_shift; l<(max_shift+1); l++)
                {
                    critere[l+max_shift]=0.0;

                    for (j=max_shift; j<(raster_col-max_shift); j++)
                    {


                        temp_diff=vect_x_active[j-l]-vect_x_ref[j];


                        critere[(l+max_shift)]+=sqrt(temp_diff*temp_diff);
                    }
                    }

                           // for each plan, the weakest criterion is chosen
            crit_min=critere[0];
            l_min=-max_shift;

            for (l=-max_shift; l<(max_shift+1); l++)
            {

                                    if (critere[(l+max_shift)]<crit_min)
                    {
                    crit_min=critere[(l+max_shift)];
                    l_min=l;
                    }
                }
                 shift_y[m]=l_min;

                  gtk_progress_bar_set_fraction(progress, (float)((float)m/((float)(nbplan-1))));
                gtk_main_iteration();
        }

// calculation of sums
   // for idmass
   for (n=0; n<nbmass; n++)
   {
               
        for (i=0; i<raster_lin; i++)
        {
            for (j=0; j<raster_col; j++)
            {
                temp_pix=0;
                
                for (m=0; m<nbplan; m++)
                    {
                    if (((i-shift_x[m])<0) || ((i-shift_x[m])>(raster_lin-1)) || ((j-shift_y[m])<0) || ((j-shift_y[m])>(raster_col-1)))
                    {
                    // nothing !
                    }
                    else {
                    temp_pix+=image[m*raster_col*raster_lin*nbmass+n*raster_col*raster_lin+(i-shift_x[m])*raster_col+j-shift_y[m]];
                    }
                    }
                image[nbplan*raster_col*raster_lin*nbmass+n*raster_col*raster_lin+(i)*raster_col+j]=temp_pix;
            }
        }

   }
file_out=fopen("shift.txt", "w");
for (m=0; m<nbplan; m++)
{
    fprintf(file_out, "%d\t%d\n", shift_x[m], shift_y[m]);
}
	fclose(file_out);
        free(shift_x);
	free(shift_y);
	free(critere);
	free(vect_x_ref);
	free(vect_x_active) ;
	free(vect_y_ref) ;
	free(vect_y_active);

        //end
        printf("Sum calculated\n");
        }
    }
    gtk_grab_remove(progress);
    return;
}

static void clicsum_test (GtkWidget *wid, GtkWidget *win)
{
    if (recal_meth==1)
    {
clicsum_SMC();
    }
if (recal_meth==2)
    {
clicsum_histogramme();
    }
if (recal_meth==3)
    {
    clicsum_alt();
    }

return;
}
static void clicboitealt (GtkWidget *wid, GtkWidget *win)
{
    int id_alt;
    char param_txt[10];

    id_alt = gtk_combo_box_get_active(GTK_COMBO_BOX(boitealt));
    if (id_alt==0)
    {
        sprintf(param_txt, "%d", filtretype);
        gtk_entry_set_text(GTK_ENTRY(paramalt), param_txt);
        gtk_entry_set_text(GTK_ENTRY(diralt1), "std_rank");
        gtk_entry_set_text(GTK_ENTRY(diralt2), "alt_rank");
    }
    if (id_alt==1)
    {
        sprintf(param_txt, "%d", (int)gtk_adjustment_get_value(GTK_ADJUSTMENT(adjust1)));
        gtk_entry_set_text(GTK_ENTRY(paramalt), param_txt);
        gtk_entry_set_text(GTK_ENTRY(diralt1), "std_anal_thresh");
        gtk_entry_set_text(GTK_ENTRY(diralt2), "alt_anal_thresh");
    }
    if (id_alt==2)
    {
        sprintf(param_txt, "%d", (int)gtk_adjustment_get_value(GTK_ADJUSTMENT(adjust2)));
        gtk_entry_set_text(GTK_ENTRY(paramalt), param_txt);
        gtk_entry_set_text(GTK_ENTRY(diralt1), "std_ref_thresh");
        gtk_entry_set_text(GTK_ENTRY(diralt2), "alt_ref_thresh");

    }
    if (id_alt==3)
    {
        gtk_entry_set_text(GTK_ENTRY(paramalt), "0.0037");
        gtk_entry_set_text(GTK_ENTRY(diralt1), "average");
        gtk_entry_set_text(GTK_ENTRY(diralt2), "natural");
    }
    if (id_alt==4)
    {
        gtk_entry_set_text(GTK_ENTRY(paramalt), "0.1");
        gtk_entry_set_text(GTK_ENTRY(diralt1), "std_2ndclass");
        gtk_entry_set_text(GTK_ENTRY(diralt2), "alt_2ndclass");
    }
    return;
}

static void save (GtkWidget *wid, GtkWidget *win)
{
    int nbconfig=1;   //number of configuration to save
    int i,j, u;
    int id_im_num;          //id determined from the numerator box of choice
    char fich_num[300];     //name of the numerator file to save from nommass[] and sPath
    int id_im_denom;        //id determined from the denominator box of choice
    char fich_denom[300];     //name of the denominator file to save from nommass[] and sPath
    int id_im_norm;        //id determined from the norm box of choice
    char fich_norm[300];     //name of the norm file to save from nommass[] and sPath
    char fich_config[300];
    char path_save[300];
    char *nom_temp;
    GtkTextBuffer *pTextBuffer; //buffer containing the script
    GtkTextIter iEnd;
    pTextBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pText_view));
    gtk_text_buffer_get_end_iter(pTextBuffer, &iEnd);

    FILE *config;
    int ok;
    FILE *im;
    float moyenne;             //input value
    id_im_num = gtk_combo_box_get_active(GTK_COMBO_BOX(choixnum));
    printf("Id num=%d\n", id_im_num);
    id_im_denom = gtk_combo_box_get_active(GTK_COMBO_BOX(choixdenom));
    printf("Id denom=%d\n", id_im_denom);
    id_im_norm = gtk_combo_box_get_active(GTK_COMBO_BOX(choixnorme));
    printf("Id norme=%d\n", id_im_norm);
    moyenne = atof(gtk_entry_get_text(GTK_ENTRY(boitemoy)));
    printf("Moyenne=%f\n", moyenne);

    if (id_im_num==-1 || id_im_denom==-1 || id_im_norm==-1 || gtk_combo_box_get_active(GTK_COMBO_BOX(choixplan))==-1)
    {
        
        printf("Incomplete selection, nothing will be saved\n");
    }
    else
    {
        nom_temp= (char*) calloc(60, sizeof(char));

        if (plan_actif==nbplan)
        {
        sprintf(nom_temp, "%s_plane_sum", gtk_entry_get_text(GTK_ENTRY(boitenomfich)));
        }
        else
        {
        sprintf(nom_temp, "%s_p%d", gtk_entry_get_text(GTK_ENTRY(boitenomfich)), plan_actif+1);
        }

        sprintf(fich_num, "%s_%s.txt", nom_temp, nom_mass[id_im_num]);
        printf("fich_num=%s\n", fich_num);
        sprintf(fich_denom, "%s_%s.txt", nom_temp, nom_mass[id_im_denom]);
        printf("fich_denom=%s\n", fich_denom);
        sprintf(fich_norm, "%s_%s.txt", nom_temp, nom_mass[id_im_norm]);
        printf("fich_norm=%s\n", fich_norm);

        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkalt)))
        {
            nbconfig=2;
        }

        for (u=0; u<nbconfig; u++)
        {

            if (strcmp(sPath, ""))
            {

                if (plan_actif==nbplan)
                {
                sprintf(path_save, "%s_plane_sum", sPath);
                }
                else
                {
                sprintf(path_save, "%s_p%d", sPath, plan_actif+1);
                }
                printf("creation de %s ", path_save);
                ok = mkdir (path_save, S_IRWXU);
                if (ok==0)
                {
                    printf("réussie\n");
                }
                if (ok==-1)
                {
                    printf("ratée. errno:%d\n", errno);
                    if (errno==17)
                    {
                        printf("Le dossier existe déjà\n");
                    }
                }
                gtk_text_buffer_insert(pTextBuffer, &iEnd, "cd ",-1);
                gtk_text_buffer_insert(pTextBuffer, &iEnd, path_save,-1);
                chdir(path_save);
                if (nbconfig==2)
                {
                    if (u==0)
                    {
                        mkdir(gtk_entry_get_text(GTK_ENTRY(diralt1)), S_IRWXU);
                        chdir(gtk_entry_get_text(GTK_ENTRY(diralt1)));
                        gtk_text_buffer_insert(pTextBuffer, &iEnd, "/",-1);
                        gtk_text_buffer_insert(pTextBuffer, &iEnd, gtk_entry_get_text(GTK_ENTRY(diralt1)),-1);

                    }
                    if (u==1)
                    {
                        mkdir(gtk_entry_get_text(GTK_ENTRY(diralt2)), S_IRWXU);
                        chdir(gtk_entry_get_text(GTK_ENTRY(diralt2)));
                        gtk_text_buffer_insert(pTextBuffer, &iEnd, "/",-1);
                        gtk_text_buffer_insert(pTextBuffer, &iEnd, gtk_entry_get_text(GTK_ENTRY(diralt2)),-1);

                    }
                }
                gtk_text_buffer_insert(pTextBuffer, &iEnd, "\nmpiexec -np 8  SIED config.ini\n\n",-1);
          }
            sprintf(fich_config, "config.ini");
            printf("fich_config=%s\n", fich_config);
            config = fopen(fich_config, "w");
            fflush(config);
            fprintf(config, "%d\n", raster_lin);
            fprintf(config, "%d\n", raster_col); // for SIED 1.7, both raster_line and raster_col are required

            if (((u*nbconfig)==2)*((gtk_combo_box_get_active(GTK_COMBO_BOX(boitealt)))==0))
            {
                fprintf(config, "%s\n", gtk_entry_get_text(GTK_ENTRY(paramalt)));
            }
            else
            {
                fprintf(config, "%d\n", filtretype);
            }
            fprintf(config, "%s\n", fich_num);
            fprintf(config, "%s\n", fich_denom);

            if (((u*nbconfig)==2)*((gtk_combo_box_get_active(GTK_COMBO_BOX(boitealt)))==1))
            {
                fprintf(config, "%s\n", gtk_entry_get_text(GTK_ENTRY(paramalt)));
            }
            else
            {
                fprintf(config, "%d\n", (int)gtk_adjustment_get_value (GTK_ADJUSTMENT(adjust1)));
            }

            if (((u*nbconfig)==2)*((gtk_combo_box_get_active(GTK_COMBO_BOX(boitealt)))==2))
            {
                fprintf(config, "%s\n", gtk_entry_get_text(GTK_ENTRY(paramalt)));
            }
            else
            {
                fprintf(config, "%d\n", (int)gtk_adjustment_get_value (GTK_ADJUSTMENT(adjust2)));
            }

            fprintf(config, "%s\n", fich_norm);

            if (((u*nbconfig)==2)*((gtk_combo_box_get_active(GTK_COMBO_BOX(boitealt)))==3))
            {
                fprintf(config, "%s\n", gtk_entry_get_text(GTK_ENTRY(paramalt)));
            }
            else
            {
                fprintf(config, "%s\n", gtk_entry_get_text(GTK_ENTRY(boitemoy)));
            }

            if (((u*nbconfig)==2)*((gtk_combo_box_get_active(GTK_COMBO_BOX(boitealt)))==4))
            {
                fprintf(config, "%s\n", gtk_entry_get_text(GTK_ENTRY(paramalt)));
            }
            else
            {
                fprintf(config, "%s\n", gtk_entry_get_text(GTK_ENTRY(seuil_proba)));
            }


            fprintf(config, "%d\n", two_tailed);

            fprintf(config, "%s\n", nom_temp);
            fclose(config);

            im = fopen(fich_num, "w");
            fflush(im);
            for (i=0; i<raster_lin; i++)
            {
                for (j=0; j<raster_col; j++)
                {
                    fprintf(im, "%d\t", image[plan_actif*raster_col*raster_lin*nbmass+id_im_num*raster_col*raster_lin+i*raster_col+j]);
                }
                fprintf(im, "\n");
            }
            fclose(im);

            im = fopen(fich_denom, "w");
            fflush(im);
            for (i=0; i<raster_lin; i++)
            {
                for (j=0; j<raster_col; j++)
                {
                    fprintf(im, "%d\t", image[plan_actif*raster_col*raster_lin*nbmass+id_im_denom*raster_col*raster_lin+i*raster_col+j]);
                }
                fprintf(im, "\n");
            }
            fclose(im);

            im = fopen(fich_norm, "w");
            fflush(im);
            for (i=0; i<raster_lin; i++)
            {
                for (j=0; j<raster_col; j++)
                {
                    fprintf(im, "%d\t", image[plan_actif*raster_col*raster_lin*nbmass+id_im_norm*raster_col*raster_lin+i*raster_col+j]);
                }
                fprintf(im, "\n");
            }
            fclose(im);
        }
        free(nom_temp);
        printf("save!\n");
    }
   return;
}
static void about (GtkWidget *wid, GtkWidget *win)
{
    GtkWidget *dialog = NULL;

    dialog = gtk_message_dialog_new (GTK_WINDOW (win), GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, "SIED_Maker 2.32\nDelaune A, Ripoll C, Cabin-Flaman A\nLaboratoire GPM UMR CNRS 6634 Equipe NanoCARE Université de Rouen\n\nanthony.delaune@univ-rouen.fr");
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
}
static void radiorecal_set_1(GtkWidget *wid, GtkWidget *win)
{
    recal_meth=1;
    return;
}
static void radiorecal_set_2(GtkWidget *wid, GtkWidget *win)
{
    recal_meth=2;
    return;
}
static void radiorecal_set_3(GtkWidget *wid, GtkWidget *win)
{
    recal_meth=3;
    return;
}




int main (int argc, char *argv[])
{
    GtkWidget *button_about = NULL;

    GtkWidget *win = NULL;
    GtkWidget *vbox = NULL;
    GtkWidget *hbox_main = NULL;
    GtkWidget *vbox2 = NULL;
    GtkWidget *vbox3 = NULL;
    GtkWidget *hbox1 = NULL;
    GtkWidget *hbox2 = NULL;
    GtkWidget *hbox3 = NULL;
    GtkWidget *hbox4 = NULL;
    GtkWidget *hbox5 = NULL;
    GtkWidget *hbox6 = NULL;
    GtkWidget *hbox7 = NULL;
    GtkWidget *hbox8 = NULL;
    GtkWidget *hbox9 = NULL;
    GtkWidget *hbox10 = NULL;
    GtkWidget *radiorecal1 = NULL;
GtkWidget *radiorecal2 = NULL;
GtkWidget *radiorecal3 = NULL;

    GtkTextBuffer *pTextBuffer; //buffer containing the script
    GtkTextIter iStart;
    GtkWidget *pScrolled; //script window scroll 

// 1 byte is arbitrarily allocated to the image
// free memory before opening images
// free memory at the end of the program
    image = (unsigned int*) calloc(1, sizeof(unsigned int));
    image_sum = (unsigned int*) calloc(1, sizeof(unsigned int));
    sPath = (char*) calloc(1, sizeof(char));

    /* Initialize GTK+ */
    g_log_set_handler ("Gtk", G_LOG_LEVEL_WARNING, (GLogFunc) gtk_false, NULL);
    gtk_init (&argc, &argv);
    g_log_set_handler ("Gtk", G_LOG_LEVEL_WARNING, g_log_default_handler, NULL);

    /* Create the main window */
    win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (win), 8);
    gtk_window_set_title (GTK_WINDOW (win), "SIED_maker 2.32");
    gtk_window_set_position (GTK_WINDOW (win), GTK_WIN_POS_CENTER);
    gtk_window_set_default_size (GTK_WINDOW (win), 1000, 500);
    gtk_widget_realize (win);
    g_signal_connect (win, "destroy", gtk_main_quit, NULL);

    /* horizontal box */
    hbox_main = gtk_hbox_new(TRUE, 4);
    gtk_container_add (GTK_CONTAINER (win), hbox_main);
    /* Create a vertical box with buttons */
    vbox = gtk_vbox_new (FALSE, 4);
    gtk_container_add (GTK_CONTAINER (hbox_main), vbox);

    pScrolled = gtk_scrolled_window_new(NULL, NULL);
    gtk_box_pack_start(GTK_BOX(hbox_main), pScrolled, TRUE, TRUE, 4);
    pText_view=gtk_text_view_new();
    gtk_container_add (GTK_CONTAINER (pScrolled), pText_view);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pScrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    hbox1 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox1);
    hbox10 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox10);


    hbox9 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox9);

    hbox2 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox2);
    hbox4 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox4);
    hbox5 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox5);
    hbox6 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox6);
    hbox7 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox7);
    hbox8 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox8);
    hbox3 = gtk_hbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (vbox),hbox3);
    vbox2 = gtk_vbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (hbox3),vbox2);
    vbox3 = gtk_vbox_new (TRUE, 4);
    gtk_container_add (GTK_CONTAINER (hbox3),vbox3);

    button_about = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
    g_signal_connect (button_about, "clicked", gtk_main_quit, NULL);
    gtk_box_pack_start (GTK_BOX (vbox), button_about, TRUE, TRUE, 0);

    button_about = gtk_button_new_from_stock (GTK_STOCK_SAVE);
    g_signal_connect (button_about, "clicked", G_CALLBACK (save), NULL);
    gtk_box_pack_start (GTK_BOX (vbox), button_about, TRUE, TRUE, 0);

    button_about = gtk_button_new_from_stock (GTK_STOCK_OPEN);
    g_signal_connect (button_about, "clicked", G_CALLBACK (open_im), NULL);
    gtk_box_pack_start (GTK_BOX(hbox1), button_about, TRUE, TRUE, 0);

    button_about = gtk_button_new_from_stock (GTK_STOCK_DIALOG_INFO);
    g_signal_connect (G_OBJECT (button_about), "clicked", G_CALLBACK (about), (gpointer) win);
    gtk_box_pack_start (GTK_BOX (hbox1), button_about, TRUE, TRUE, 0);

    progress= gtk_progress_bar_new();
    gtk_box_pack_start (GTK_BOX (vbox), progress, TRUE, TRUE, 0);




    choixplan = gtk_combo_box_new_text ();
    choixnum = gtk_combo_box_new_text ();
    choixdenom = gtk_combo_box_new_text ();
    choixnorme = gtk_combo_box_new_text ();
    boitealt = gtk_combo_box_new_text ();

    checksum = gtk_button_new_with_label("Sum the planes (autotrack)");
    g_signal_connect (G_OBJECT (checksum), "clicked", G_CALLBACK (clicsum_test), (gpointer) win);
    gtk_box_pack_start (GTK_BOX (hbox9), checksum, TRUE, TRUE, 0);

    radiorecal1 = gtk_radio_button_new_with_label(NULL, "Least squ. meth.");
    radiorecal2 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radiorecal1), "Joint hist. meth.");
    radiorecal3 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radiorecal2), "1D proj. meth.");
    g_signal_connect (G_OBJECT (radiorecal1), "clicked", G_CALLBACK (radiorecal_set_1), (gpointer) win);
    g_signal_connect (G_OBJECT (radiorecal2), "clicked", G_CALLBACK (radiorecal_set_2), (gpointer) win);
    g_signal_connect (G_OBJECT (radiorecal3), "clicked", G_CALLBACK (radiorecal_set_3), (gpointer) win);
    gtk_box_pack_start (GTK_BOX (hbox9), radiorecal1, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (hbox9), radiorecal2, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (hbox9), radiorecal3, TRUE, TRUE, 2);

    button_about = gtk_label_new("Shift max");
    gtk_box_pack_start (GTK_BOX (hbox9), button_about, TRUE, TRUE, 0);
    paramsum = gtk_spin_button_new_with_range(0, 4, 1);
    gtk_box_pack_start (GTK_BOX (hbox9), paramsum, TRUE, TRUE, 0);
    gtk_spin_button_set_value(paramsum, 2);
    choixref = gtk_combo_box_new_text ();
    gtk_box_pack_start (GTK_BOX (hbox9), choixref, TRUE, TRUE, 0);

//image display window
    pwindow = gtk_window_new(GTK_WINDOW_POPUP); 
    pVBoxmain=gtk_vbox_new(FALSE, 0);

    gtk_container_add(GTK_CONTAINER(pwindow), pVBoxmain);
    pImage = gtk_image_new();
    gtk_box_pack_start(GTK_BOX(pVBoxmain), pImage, TRUE, TRUE, 0);

    g_signal_connect (choixplan, "changed", G_CALLBACK (change_plan), NULL);
    g_signal_connect (choixdenom, "changed", G_CALLBACK (change_denom), NULL);

    button_about = gtk_label_new("Active plane");
    gtk_box_pack_start (GTK_BOX (hbox2), button_about, TRUE, TRUE, 0);
    button_about = gtk_label_new("Label mass");
    gtk_box_pack_start (GTK_BOX (hbox2), button_about, TRUE, TRUE, 0);
    button_about = gtk_label_new("Non-label mass");
    gtk_box_pack_start (GTK_BOX (hbox2), button_about, TRUE, TRUE, 0);
    button_about = gtk_label_new("Add. mass");
    gtk_box_pack_start (GTK_BOX (hbox2), button_about, TRUE, TRUE, 0);

    gtk_box_pack_start (GTK_BOX (hbox4), choixplan, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox4), choixnum, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox4), choixdenom, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox4), choixnorme, TRUE, TRUE, 0);

    checkmoyenne = gtk_check_button_new_with_label("Set reference value");
    g_signal_connect (G_OBJECT (checkmoyenne), "clicked", G_CALLBACK (clicmoy), (gpointer) win);
    gtk_box_pack_start (GTK_BOX (vbox2), checkmoyenne, TRUE, TRUE, 0);

    boitemoy =  gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(boitemoy), "-0.001");
    gtk_box_pack_start (GTK_BOX (vbox3), boitemoy, TRUE, TRUE, 0);

    button_about = gtk_label_new("Analysis threshold");
    gtk_box_pack_start (GTK_BOX (vbox3), button_about, TRUE, TRUE, 0);

//   denominator threshold selection slider
    adjust1 = gtk_adjustment_new (1, 1, 1000+50, 1, 10, 50);
    adjust2 = gtk_adjustment_new (1, 0, 1000+50, 1, 10, 50);

    choix_seuil_denom = gtk_hscale_new (GTK_ADJUSTMENT(adjust1));
    gtk_scale_set_digits(GTK_SCALE(choix_seuil_denom), 0);
    g_signal_connect (G_OBJECT (choix_seuil_denom), "value_changed", G_CALLBACK (change_seuil_denom), (gpointer) win);  //signal
    gtk_box_pack_start (GTK_BOX (vbox3), choix_seuil_denom, TRUE, TRUE, 0);

    button_about = gtk_label_new("Reference threshold");
    gtk_box_pack_start (GTK_BOX (vbox3), button_about, TRUE, TRUE, 0);

//  average threshold selection slider
    choix_seuil_moy = gtk_hscale_new(GTK_ADJUSTMENT(adjust2));
    gtk_scale_set_digits(GTK_SCALE(choix_seuil_moy), 0);
    g_signal_connect (G_OBJECT (choix_seuil_moy), "value_changed", G_CALLBACK (change_seuil_moy), (gpointer) win);  //signal
    gtk_box_pack_start (GTK_BOX (vbox3), choix_seuil_moy, TRUE, TRUE, 0);

    radio4 = gtk_radio_button_new_with_label(NULL, "Matrix rank 3 : 49 pixels");
    radio1 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio4), "Matrix rank 0 : 1 pixel");
    radio2 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio1), "Matrix rank 1 : 9 pixels");
    radio3 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio2), "Matrix rank 2 : 25 pixels");
    radio5 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio3), "Matrix rank 4 : 81 pixels");

    g_signal_connect (G_OBJECT (radio1), "clicked", G_CALLBACK (radio_set_1), (gpointer) win);
    g_signal_connect (G_OBJECT (radio2), "clicked", G_CALLBACK (radio_set_2), (gpointer) win);
    g_signal_connect (G_OBJECT (radio3), "clicked", G_CALLBACK (radio_set_3), (gpointer) win);
    g_signal_connect (G_OBJECT (radio4), "clicked", G_CALLBACK (radio_set_4), (gpointer) win);
    g_signal_connect (G_OBJECT (radio5), "clicked", G_CALLBACK (radio_set_5), (gpointer) win);

    gtk_box_pack_start (GTK_BOX (vbox2), radio1, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox2), radio2, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox2), radio3, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox2), radio4, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox2), radio5, TRUE, TRUE, 2);

    radio2t1 = gtk_radio_button_new_with_label(NULL, "One-tailed test (enrichment detection)");
    radio2t2 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio2t1), "Two-tailed test (heterogeneity study)");
    g_signal_connect (G_OBJECT (radio2t1), "clicked", G_CALLBACK (radio2t_set_1), (gpointer) win);
    g_signal_connect (G_OBJECT (radio2t2), "clicked", G_CALLBACK (radio2t_set_2), (gpointer) win);
    gtk_box_pack_start (GTK_BOX (hbox10), radio2t1, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (hbox10), radio2t2, TRUE, TRUE, 2);


    button_about = gtk_check_button_new_with_label("Change 2nd class risk(NOT recommended)");
    g_signal_connect (G_OBJECT (button_about), "clicked", G_CALLBACK (changeproba), (gpointer) win);
    gtk_box_pack_start (GTK_BOX (vbox2), button_about, TRUE, TRUE, 0);
    seuil_proba = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(seuil_proba), "0.1");
    gtk_box_pack_start (GTK_BOX (vbox3), seuil_proba, TRUE, TRUE, 0);

    button_about = gtk_label_new("Analysis prefix");
    gtk_box_pack_start (GTK_BOX (hbox5), button_about, TRUE, TRUE, 0);

    boitenomfich = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(boitenomfich), "");
    gtk_box_pack_start (GTK_BOX (hbox5), boitenomfich, TRUE, TRUE, 0);

    checkalt = gtk_check_button_new_with_label("Alternate analysis");
    g_signal_connect (G_OBJECT (checkalt), "clicked", G_CALLBACK (clicalt), (gpointer) win);
    gtk_box_pack_start (GTK_BOX (hbox8), checkalt, TRUE, TRUE, 0);
    paramalt =  gtk_entry_new();
    diralt1 =  gtk_entry_new();
    diralt2 =  gtk_entry_new();
    button_about = gtk_label_new("Standard directory");
    gtk_box_pack_start (GTK_BOX (hbox6), button_about, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox6), diralt1, TRUE, TRUE, 0);
    button_about = gtk_label_new("Alternative directory");
    gtk_box_pack_start (GTK_BOX (hbox7), button_about, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox7), diralt2, TRUE, TRUE, 0);

    gtk_combo_box_append_text(GTK_COMBO_BOX(boitealt), "Matrix rank");
    gtk_combo_box_append_text(GTK_COMBO_BOX(boitealt), "Anal.thresh");
    gtk_combo_box_append_text(GTK_COMBO_BOX(boitealt), "Ref.thresh");
    gtk_combo_box_append_text(GTK_COMBO_BOX(boitealt), "Reference");
    gtk_combo_box_append_text(GTK_COMBO_BOX(boitealt), "2ndclass_risk");

    gtk_box_pack_start (GTK_BOX (hbox8), boitealt, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox8), paramalt, TRUE, TRUE, 0);
    g_signal_connect (G_OBJECT (boitealt), "changed", G_CALLBACK (clicboitealt), (gpointer) win);

pTextBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pText_view));
gtk_text_buffer_get_start_iter(pTextBuffer, &iStart);
gtk_text_buffer_insert(pTextBuffer, &iStart, "#!/bin/sh\n\nDIR=$( cd \"$( dirname \"${BASH_SOURCE[0]}\" )\" && pwd )\n\n", -1);

    /* Enter the main loop */
    gtk_widget_show_all (win);
    gtk_widget_hide(boitemoy);
    gtk_widget_hide(seuil_proba);
    gtk_widget_hide(boitealt);
    gtk_widget_hide(paramalt);
    gtk_widget_hide(diralt1);
    gtk_widget_hide(diralt2);
    gtk_main ();
    free(image); //free memory from the image
    free(image_sum);
    free(impix);
    free(sPath);
    return 0;
}
